#ifndef __Blackboard__
#define __Blackboard__

enum BlackboardObjectTypes { Boolean,Scalar,DoubleScalar,Vector,DoubleVector,Matrix,DoubleMatrix,Raw };

#define Interpolation_Linear 0x00

class Blackboard {
 public:
  Blackboard(void);
  ~Blackboard(void);

  void Initialize(void);
  void Free(void);


  //Network connection related stuff
  int ConnectToServer(void);
  int ConnectToServer(const char * address,int port = SADBPort);
  void ConnectToServerWithRetry(void);
  void ConnectToServerWithRetry(const char * address,int port = SADBPort);
  
  
  void GenerateHeader(SADBu8* Header,SADBu8 CMD,SADBu64 OUID1,SADBu64 OUID2,SADBu64 Second,SADBu16 Millisecond,SADBu32 DataLength);
  void Disconnect(void);

 
  //SADB Commands
  SADBs64 Ping(void);
  SADBu64 CreateObject(void);
  void DeleteObject(SADBu64 OUID);
  int IsValidObject(SADBu64 OUID);
  void SetObjectName(SADBu64 OUID,const char * Name);
  void GetObjectName(SADBu64 OUID,SADBu8 ** Name);
  void SetObjectDescription(SADBu64 OUID,const char * Description);
  void GetObjectDescription(SADBu64 OUID,SADBu8 ** Description);
  SADBu64 GetObjectByName(const char * Name);
  int SendObjectValue(SADBu64 OUID, BlackboardObject * Object);
  int GetLatestObjectValue(SADBu64 OUID, BlackboardObject * Object);
  int GetLatestObjectTimestamp(SADBu64 OUID, Timestamp * time);
  int GetNextObjectTimestamp(SADBu64 OUID,Timestamp*Current,Timestamp*Next);
  int GetPreviousObjectTimestamp(SADBu64 OUID,Timestamp*Current,Timestamp*Previous);
  int GetNearestObjectTimestamp(SADBu64 OUID,Timestamp*Target,Timestamp*Result);
  
  int GetObjectValueAtTimestamp(SADBu64 OUID, BlackboardObject * Object,Timestamp * Time,int Interpolation,int nPoints);
  int GetObjectValueAtNearestTimestamp(SADBu64 OUID,Timestamp * Time, BlackboardObject * Object);
  int RemoveOldestValue(SADBu64 OUID);
  int KeepOnlyLatestValues(SADBu64 OUID,int nValues);

  int GetObjectValueAtNextTimestamp(SADBu64 OUID,Timestamp * Time, BlackboardObject * Object);
  int GetObjectValueAtPreviousTimestamp(SADBu64 OUID,Timestamp * Time, BlackboardObject * Object);

  SADBu64 CreateCategory(void);
  SADBu64 CreateCategoryIfNonExistant(const char * Name);
  void DeleteCategory(SADBu64 CUID);
  int IsValidCategory(SADBu64 CUID);
  void SetCategoryName(SADBu64 CUID,const char * Name);
  void GetCategoryName(SADBu64 CUID,SADBu8 ** Name);
  void SetCategoryDescription(SADBu64 CUID,const char * Description);
  void GetCategoryDescription(SADBu64 CUID,SADBu8 ** Description);
  SADBu64 GetCategoryByName(const char * Name);
  void AddObjectToCategory(SADBu64 CUID,SADBu64 OUID);
  void RemoveObjectFromCategory(SADBu64 CUID,SADBu64 OUID);
  int FindObjects(const char *Request,SADBu64**Objects);

  SADBu64 GetNextObjectID(void);
  SADBu64 GetNextCategoryID(void);


  void Reset(void);
  
  char * Address;
  int Port;
 private:
  Network SADB;
};
#endif 

