#ifndef __BlackboardObject__
#define __BlackboardObject__

class Blackboard;

class BlackboardObject {
 public:
  BlackboardObject(void);
  ~BlackboardObject(void);
  void Initialize(void);
  void Free(void);

  //Generalized Constructor (planned)
  void Create(Blackboard * board,const char * name);
  void Create(Blackboard * board,const char * name,const char * description);
  void Create(Blackboard * board,const char * name,const char * description,enum BODataType type);
  void Create(Blackboard * board,const char * name,enum BODataType);

  //SADB Interface functions
  void SetBlackboard(Blackboard * board);
  void AddToBlackboard(void);
  int Send(void);

  void AddToCategory(SADBu64 CUID);
  void AddToCategory(const char * Cat);
  void RemoveFromCategory(SADBu64 CUID);
  void RemoveFromCategory(const char * Cat);

  void CreateIfNonExistant(const char * name);

  void GetByName(const char * name);
  int GetLatestValue(void);
  int GetValueAtNearestTimestamp(Timestamp * Time);
  int GetValueAtTimestampNoInterpolation(Timestamp * Time);
  int GetValueAtTimestamp(Timestamp * Time);
  int GetValueAtTimestamp(Timestamp * Time,int Interpolation);
  int GetValueAtTimestamp(Timestamp * Time,int Interpolation,int nPoints);
  int GetValueAtNextTimestamp(Timestamp * Time);
  int GetValueAtPreviousTimestamp(Timestamp * Time);

  int GetLatestTimestamp(Timestamp * Time);
  int GetPreviousTimestamp(Timestamp * Current,Timestamp*Previous);
  int GetNextTimestamp(Timestamp * Current,Timestamp*Next);
  int GetNearestTimestamp(Timestamp * Target,Timestamp*Result);
  
  void SetDataType(enum BODataType);

  int SetDimensionsf(SADBu32 D,...);
  int OnlySetDimensionsf(SADBu32 D,...);
  int SetDimensions(SADBu32 D,SADBu32*Ds);
  SADBu32 GetDimension(SADBu32 D);

  int SetName(const char * name);
  void GetName(void);
  int SetDescription(const char * desc);
  void GetDescription(void);

  void CreateWithInitialValueFromCSV(Timestamp * Time,const char * Filename,enum BODataType DataType,char Delimiter);
  void CreateWithInitialValueFromCSV(const char * Filename,enum BODataType DataType);
  void CreateWithInitialValueFromCSV(const char * Filename,enum BODataType DataType,char Delimiter);
  void CreateWithValuesFromCSV(const char * Filename,enum BODataType DataType);
  void CreateWithValuesFromCSV(const char * Filename,enum BODataType DataType,char Delimiter);

  void ExportAllValuesToCSV(const char * Filename);

  void ExportRaw(const char * Filename);
  void ImportRaw(const char * Filename);
  void ImportRaw(const char * Filename,float A,float B);
  
  void ExportAscii(const char * Filename);
  void ImportAscii(const char * Filename);
//Accessors

  void SetByte(SADBu8 Value,...);
  void SetInteger(SADBu32 Value,...);
  void SetFloat(SADBfl Value,...);
  void SetDouble(SADBdfl Value,...);
  void SetString(const char * str);
  void SetStringElement(const char * str,...);
  void SetBoolean(bool Value,...);
 
  SADBu8 GetByte(int X,...);
  SADBu32 GetInteger(int X,...);
  SADBfl GetFloat(int X,...);
  SADBdfl GetDouble(int X,...);
  const char * GetString(void);
  char * GetNewString(void);
  bool GetBoolean(int X,...);
  char * GetStringElement(int X,...);







 void RemoveOldestValue(void);
 void KeepOnlyLatestValues(int nValues);


 int CheckForNewValue(void);


  void Information(void);
  void SetTimestamp(Timestamp * time);
  void SetTimestampNow(void);

  /// Blackboard where the object resides
  Blackboard * Board;
   
  /// Human Readable Object Name
  SADBu8 * Name;
  /// Human Readable Object Description
  SADBu8 * Description;
  /// Pointer to Memory Location of the data
  void * Data;
  /// Dimensionality of the data (number of dimensiosns)
  SADBu32 Dimensionality;
  /// Dimensions of the data
  SADBu32 * Dimensions;
  /// Total number of elements (Product of all dimensions)
  int TotalNumberOfElements;
  /// Data Type
  enum BODataType DataType;
  /// Element Size (in bytes)
  int ElementSize;
  /// Object ID on the blackboard
  SADBu64 ID;
  /// Timestamp of the data
  Timestamp Time;


 private:
  Pointer GetElementLocation(int*Pos);

};
#endif 

