#include "SADB.h"
using namespace SADB;

/// Constructor, initializes object
/// @see Blackboard::Initialize
BlackboardObjectListener::BlackboardObjectListener(void){}

/// Destructor, deinitializes object
/// @see Blackboard::Free
BlackboardObjectListener::~BlackboardObjectListener(void){Free();}

/// Initializes object
/// @param Obj
///  Object to be periodically checked
/// @param period
///  Period of the check (in us)
/// @param callback
///  Callback function, which should have prototype "void functionname(BlackboardObject *)
void BlackboardObjectListener::Initialize(BlackboardObject * Obj,SADBu64 period,void(*callback)(BlackboardObject *)){
 Target=Obj;
 Period=period;
 Callback=callback;
}

/// Frees allocated memory
void BlackboardObjectListener::Free(void){
}


/// Start the main thread for the Object Listener
void BlackboardObjectListener::Start(void){
 StopRequest=0;
 if(pthread_create(&ThreadID,NULL,StartMainThreadProxy,this))Error("BlackboardObjectListener","Cannot start thread");
}

/// Stop the main thread for the Object Listener
void BlackboardObjectListener::Stop(void){
 StopRequest=1;
}

/// Static function to start the main thread
/// @param context
///   void casted pointer to a BlackboardObjectListener object
void *BlackboardObjectListener::StartMainThreadProxy(void *context){
 ((BlackboardObjectListener *)context)->Main();//Call server main
 return NULL;
}


/// Main Function of the objectlistener thread
void BlackboardObjectListener::Main(void){
 while(StopRequest==0){
  if(Target->CheckForNewValue()==1){//A new value is available
   Target->GetLatestValue();//Get the new value
   Callback(Target);//Pass the object to the callback method
  }
  usleep(Period);//Sleep until the next check
 }
 pthread_exit(NULL);
}
