#include "SADB.h"

///Cork a socket, doesn't seem to be any equivalent in windows, could be modified if there is one
///@note Windows "hack" from http://stackoverflow.com/questions/22117205/is-there-an-equivalent-to-tcp-cork-in-winsock 
void SADB::CorkSocket(int Socket){
 #ifndef WINDOWS
 int flag=1;
 setsockopt(Socket,SOL_TCP,TCP_CORK,&flag,sizeof(flag));
 #else
 //int flag=0;setsockopt(Socket,IPPROTO_TCP,TCP_NODELAY,&flag,sizeof(flag));//Enable Naggle (prevent sending of small packets)
 #endif
}

///UnCork a socket, doesn't seem to be any equivalent in windows, could be modified if there is one
///@note Windows "hack" from http://stackoverflow.com/questions/22117205/is-there-an-equivalent-to-tcp-cork-in-winsock 
void SADB::UnCorkSocket(int Socket){
 #ifndef WINDOWS
 int flag=0;
 setsockopt(Socket,SOL_TCP,TCP_CORK,&flag,sizeof(flag));
 #else
 //int flag=1;setsockopt(Socket,IPPROTO_TCP,TCP_NODELAY,&flag,sizeof(flag));//Disable Naggle
 //send(Socket,NULL,0,0);//Force a send
 #endif
}

///Wait for a response from the server, don't care about the contents
/// @return 0 on success, 1 otherwise
int SADB::SADBWaitForResponse(int Socket){
 SADBu8 CMD;SADBu64 ID,Argument;
 SADBu32 DataLength;SADBu8 *Data=NULL;
 SADBu64 Second;SADBu16 Millisecond;
 int r=SADBReceive(Socket,&CMD,&ID,&Argument,&Second,&Millisecond,&DataLength,&Data);
 if(Data!=NULL)free(Data);//Just in case
 return r;
}


///Receive a packet and keep the string
/// @param String
///  Pointer to a string to store result.  The memory will be allocated as needed.
/// @return 0 on success, 1 otherwise
int SADB::SADBReceiveString(int Socket,SADBu8**String){
 SADBu8 CMD;SADBu64 ID,Argument;
 SADBu32 DataLength;
 SADBu64 Second;SADBu16 Millisecond;
 int r=SADBReceive(Socket,&CMD,&ID,&Argument,&Second,&Millisecond,&DataLength,String);
 return r;
}

///Receive a packet and keep the ID
/// @param String
///  Pointer to a string to store result.  The memory will be allocated as needed.
/// @return 0 on success, 1 otherwise
int SADB::SADBReceiveID(int Socket,SADBu64*ID){
 SADBu8 CMD;SADBu64 Argument;
 SADBu32 DataLength;SADBu8 *Data=NULL;
 SADBu64 Second;SADBu16 Millisecond;
 int r=SADBReceive(Socket,&CMD,ID,&Argument,&Second,&Millisecond,&DataLength,&Data);
 if(Data!=NULL)free(Data);//Just in case
 return r;
}




///Receive an SADB Packet
/// @param Header
///  Pointer to the memory location of the header
/// @param CMD
///  Command to send
/// @param OUID
///  Object Unique ID
/// @param Argument
///  Optional argument
/// @param Second
///  Timestamp of the object
/// @param Millisecond
///  Timestamp of the object
/// @param DataLength
///  Length of the data segment
/// @return 0 on success, 1 otherwise
int SADB::SADBReceive(int Socket,SADBu8 *CMD,SADBu64 *OUID,SADBu64 *Argument,SADBu64 *Second,SADBu16 *Millisecond,SADBu32 *DataLength,SADBu8 ** Data){

 SADBu8 Header[SADBHeaderLength];
 if(SocketReceiveExactly(Socket,Header,SADBHeaderLength)!=SADBHeaderLength)return 1;

 //Sanity check
 if((Header[0x00]!=0x5A)||(Header[0x01]!=0xDB)){return 1;}
 //Extract info
 *CMD=Header[0x02];
 *OUID=*((SADBu64*)(Header+0x03));
 *Argument=*((SADBu64*)(Header+0x0B));
 *Second=*((SADBu64*)(Header+0x13));
 *Millisecond=*((SADBu16*)(Header+0x1B));
 *DataLength=*((SADBu32*)(Header+0x1D));

 if(*DataLength>0){
  *Data=(SADBu8 *)malloc(*DataLength);
  if(SocketReceiveExactly(Socket,*Data,*DataLength)!=(int)*DataLength)return 1;
 }


 return 0;//success
}






/// Generate an SADB packet Header
/// @param Header
///  Pointer to the memory location of the header
/// @param CMD
///  Command to send
/// @param OUID
///  Object Unique ID
/// @param Argument
///  Optional argument
/// @param Second
///  Timestamp of the object
/// @param Millisecond
///  Timestamp of the object
/// @param DataLength
///  Length of the data segment
void SADB::GenerateSADBHeader(SADBu8* Header,SADBu8 CMD,SADBu64 OUID,SADBu64 Argument,SADBu64 Second,SADBu16 Millisecond,SADBu32 DataLength){
 Header[0x00]=0x5A;Header[0x01]=0xDB;//SADB Marker
 Header[0x02]=CMD;//Command

 *((SADBu64*)(Header+0x03))=OUID; // Object ID
 *((SADBu64*)(Header+0x0B))=Argument; // argument
 *((SADBu64*)(Header+0x13))=Second; // Timestamp.Second
 *((SADBu16*)(Header+0x1B))=Millisecond; // Timestamp.Millisecond
 *((SADBu32*)(Header+0x1D))=DataLength; // Data Length

}



/// Send an SADB packet
/// @param Socket
///  Target Socket
/// @param CMD
///  Command to send
/// @param OUID
///  Object Unique ID
/// @param Argument
///  Optional argument
/// @param Second
///  Timestamp of the object
/// @param Millisecond
///  Timestamp of the object
/// @param DataLength
///  Length of the data segment
/// @param Data
///  Pointer to the data
/// @return 0 on success, 1 otherwise
int SADB::SendSADBPacket(int Socket,SADBu8 CMD,SADBu64 OUID,SADBu64 Argument,SADBu64 Second,SADBu16 Millisecond,SADBu32 DataLength,SADBu8 * Data){
 CorkSocket(Socket);//Cork the socket
  
 //Generate and Send the header
 SADBu8 Header[SADBHeaderLength];
 GenerateSADBHeader(Header,CMD,OUID,Argument,Second,Millisecond,DataLength);
 int r=SocketSend(Socket,Header,SADBHeaderLength);
 if(r){
  UnCorkSocket(Socket);//Uncork the socket, just in case
  return 1;//failed to send
 }
 //Send the data
 if(DataLength!=0)r=SocketSend(Socket,Data,DataLength);
 
 UnCorkSocket(Socket);//Uncork the socket
 
 if(r)return 1;//failed to send
 return 0;
}

/// Send an SADB packet with 2 data blocks
/// @param Socket
///  Target Socket
/// @param CMD
///  Command to send
/// @param OUID
///  Object Unique ID
/// @param Argument
///  Optional argument
/// @param Second
///  Timestamp of the object
/// @param Millisecond
///  Timestamp of the object
/// @param DataLengthA
///  Length of the first data segment
/// @param DataA
///  Pointer to the first data location
/// @param DataLengthB
///  Length of the second data segment
/// @param DataB
///  Pointer to the sencond data location
int SADB::SendSADBPacket(int Socket,SADBu8 CMD,SADBu64 OUID,SADBu64 Argument,SADBu64 Second,SADBu16 Millisecond,SADBu32 DataLengthA,SADBu8 * DataA,SADBu32 DataLengthB,SADBu8 * DataB){
 CorkSocket(Socket);//Cork the socket
 //Generate and Send the header
 SADBu8 Header[SADBHeaderLength];
 GenerateSADBHeader(Header,CMD,OUID,Argument,Second,Millisecond,DataLengthA+DataLengthB);
 int r=SocketSend(Socket,Header,SADBHeaderLength);
 if(r){
  UnCorkSocket(Socket);//Uncork the socket, just in case
  return 1;//failed to send
 }
 //Send the data
 if(DataLengthA!=0)r=SocketSend(Socket,DataA,DataLengthA);
 if(r){
  UnCorkSocket(Socket);//Uncork the socket, just in case
  return 1;//failed to send
 }
 //Send the data
 if(DataLengthB!=0)r=SocketSend(Socket,DataB,DataLengthB);
 UnCorkSocket(Socket);//Uncork the socket
 
 return r;
}
