#ifndef __SADBStringTable__
#define __SADBStringTable__
/// Class used to store tables of strings in SADB
/// @note If you change the dimensions of the table to a smaller size (shrink), some object will remain on the blackboard without being linked to a string table.  If the table size is incremented at a later time, the objects will be linked again.  This is to preserve history.
class SADBStringTable {
 public:
  SADBStringTable(void);
  ~SADBStringTable(void);
  void Initialize(void);
  void Free(void);

  void SetName(const char * name);
  void GetByName(const char * name);
  void SetDescription(const char * desc);

  void SetBlackboard(Blackboard * board);
  void SetDimensions(int X,int Y);

  void SetString(int X,int Y,const char * Str);
  const char * GetString(int X,int Y);
  char * GetNewString(int X,int Y);

  void LoadCSV(const char * Filename);

  void GetLatestValues(void);
  void GetLatestValue(int X,int Y);


  /// Human Readable Object Name
  SADBu8 * Name;

  ///Object to store the list of indices
  BlackboardObject Object;

  ///Number of Columns
  int nCols;
  ///Number of Rows
  int nRows;

  ///Blackboard
  Blackboard * Board;
 private:
  int GetIndex(int X,int Y);
  BlackboardObject * Objects;
};
#endif 




