#include "SADB.h"
using namespace SADB;

/// Constructor
/// @see Timestamp::Initialize
Timestamp::Timestamp(void){Initialize();}

/// Destructor
/// @see Timestamp::Free
Timestamp::~Timestamp(void){Free();}

/// Initializes object to 0
void Timestamp::Initialize(void){
 Second=0;
 Millisecond=0;
}

/// Frees allocated memory
void Timestamp::Free(void){
 
}

/// Set the timestamp to the current time
void Timestamp::Now(void){
 struct timeb now;//Time since the epoch (unixtime)
 ftime(&now);
 Second=(SADBu64)now.time;
 Millisecond=(SADBu16)now.millitm;
}

/// Manually set the timestamp
/// @param second
///  Target Time (second part)
/// @param millisecond
///  Target Time (millisecond part)
void Timestamp::Set(SADBu64 second,SADBu16 millisecond){
 Second=second;
 Millisecond=millisecond;
}

/// Manually set the timestamp, in SMC double format
/// @param time
///  Target Time
void Timestamp::Set(SADBdfl time){
 double r=fmod(time,1);//Fractional part
 Millisecond=1000*r;
 Second=floor(time);
}

/// Get the timestamp in SMC double format
SADBdfl Timestamp::Get(void){
 return (SADBdfl)Second+(SADBdfl)Millisecond/1000;
}

/// Get the number of seconds between the current timestamp object and another timestamp object
/// @param B
///  The other timestamp
/// @note If you have 2 timestamp objects A and B, A.SecondDifference(&B) will return B-A.
/// @note If the distance between timestamps is very large, there is a possibility of overflow.  This should only happen if the number of seconds between the two timestamps is greater than ~ 9e15.
SADBdfl Timestamp::SecondDifference(Timestamp * B){
 SADBs64 secdiff=B->Second-Second;
 SADBs16 milldiff=B->Millisecond-Millisecond;
 return (SADBdfl)secdiff+(SADBdfl)milldiff/1000;
}

/// Get the number of seconds between the current timestamp object and another timestamp object
/// @param B
///  The other timestamp
/// @note If you have 2 timestamp objects A and B, A.SecondDifference(&B) will return B-A. 
/// @note If the distance between timestamps is very large, there is a possibility of overflow.  This should only happen if the number of seconds between the two timestamps is greater than ~ 1.8e16.
SADBs64 Timestamp::MillisecondDifference(Timestamp * B){
 SADBs64 secdiff=B->Second-Second;
 SADBs16 milldiff=B->Millisecond-Millisecond;
 return secdiff*1000+milldiff;
}

/// Print the timestamp
void Timestamp::Print(void){
 printf("%lld.%d",Second,Millisecond);
}
