#include "SADB.hpp"

//Some Basic Functions

/// Convert a u8 to the BODataType equivalent
/// @param Type
///   Integer representation of the data type
/// @return
///   enum
BODataType DataTypeIntToEnum(u8 Type){
 switch(Type){
  case 0x00: return Byte;break;
  case 0x01: return Integer;break;
  case 0x02: return Float;break;
  case 0x03: return Double;break;
  default:   return Byte;//Default to single byte
 }
}


/// Print A log Message
void Log(const char*Format,...){
 char Time[1024];
 va_list Arguments;
 va_start(Arguments,Format);
 time_t T;time(&T);
 struct tm * LocalTime=localtime(&T);
 strftime(Time,1024,"%H:%M:%S",LocalTime);
 printf("[%s] ",Time);
 vprintf(Format,Arguments);
 printf("\n");
 va_end(Arguments);
}


/// Exit program and display an error message
/// @param[in] Title
///   Error name
/// @param[in] Description
///   Error name
void Error(const char *Title,const char *Description){
 printf("###########################\n");
 printf("########!!!ERROR!!!########\n");
 printf("###########################\n");
 printf("%s\n",Title);
 printf("%s\n",Description);
 exit(0);
}

#ifdef POSIX
/// Get timestamp corresponding to current time
/// @note If not on a Posix system, the timestamp only includes seconds 
/// @param TS
///   TimeStamp structure to hold the result
void GetTimestamp(Timestamp & TS){
 struct timeval now;//Time since the epoch (unixtime)
 gettimeofday(&now,NULL);
 TS.Second=(u64)now.tv_sec;
 TS.Millisecond=(u16)(now.tv_usec/1000);
}
#endif


#ifdef WINDOWS
/// Get timestamp corresponding to current time
/// @note If not on a Posix system, the timestamp only includes seconds 
/// @param TS
///   TimeStamp structure to hold the result
void GetTimestamp(Timestamp & TS){
 TS.Second=(TimeStamp)time(NULL);
 TS.Millisecond=0;
}
#endif 

/// Get Difference between two timestamps (in milliseconds)
/// @param A
///  First Timestamp
/// @param B
///  Second Timestamp
/// @return Difference
s64 TimestampDifference(Timestamp & A,Timestamp & B){
 s64 secdiff=B.Second-A.Second;
 s16 milldiff=B.Millisecond-A.Millisecond;
 return secdiff*1000+milldiff;
}

/// Get Difference between two timestamps (in milliseconds)
/// @param A First Timestamp
/// @param B Second Timestamp
/// @return Difference
s64 TimestampDifference(Timestamp*A,Timestamp*B){
 s64 secdiff=B->Second-A->Second;
 s16 milldiff=B->Millisecond-A->Millisecond;
 return secdiff*1000+milldiff;
}

/// Convert a timestamp to milliseconds
/// @param A
///  Timestamp
/// @return Milliseconds
u64 TimestampToMilliseconds(Timestamp & A){
 return A.Second*1000+A.Millisecond;
}

/// Get Difference between two timestamps (in milliseconds)
/// @param SecondA
///  Second component of timestampA
/// @param MillisecondA
///  Millisecond component of timestampA
/// @param SecondB
///  Second component of timestampB
/// @param MillisecondB
///  Millisecond component of timestampB
/// @return Difference
s64 TimestampDifference(u64 SecondA,u16 MillisecondA,u64 SecondB,u16 MillisecondB){
 s64 secdiff=SecondB-SecondA;
 s16 milldiff=MillisecondB-MillisecondA;
 return secdiff*1000+milldiff;
}

/// Print timestamp
/// @param TS
///   TimeStamp structure to print
void PrintTimestamp(Timestamp & TS){
 printf("%lld.%d",TS.Second,TS.Millisecond);
}


