///
/// Polynomial Interpolation
/// @param Target
///  Desired TimeStamp
/// @param nPoints
///  Number of points to consider, should be even
///
void Interpolator::Polynomial(Timestamp Target,int nPoints){
 SADBObjectStorage **V=(SADBObjectStorage**)malloc(nPoints*sizeof(SADBObjectStorage*));
 if(V==NULL)return;

 if(Object->nValues>nPoints){
  if(Index<=nPoints/2-1){//from the start, need to take first nValues
   for(int i=0;i<nPoints;i++)V[i]=Object->Values[i];
  }else if(Index>=Object->nValues-nPoints/2-1){//from the end, take last nValues
   for(int i=0;i<nPoints;i++)V[i]=Object->Values[Object->nValues-i-1];
  }else{//Normal case
   for(int i=0;i<nPoints;i++)V[i]=Object->Values[Index-nPoints/2+i];
  }
 }else{//Not enough data points to interpolate
  return;
 }

 for(int i=0;i<nPoints-1;i++)if(V[i]->TotalNumberOfElements!=V[i+1]->TotalNumberOfElements)return;//Not all the same size
 int Size=V[0]->TotalNumberOfElements;

 //Compute the times (time between each value and the first, in millisecond)
 double *T=(double*)malloc(nPoints*sizeof(double));
 T[0]=0;
 for(int i=1;i<nPoints;i++){
  int Mil=V[i]->Time.Millisecond;
  int Sec=V[i]->Time.Second;
  Mil-=V[0]->Time.Millisecond;
  if(Mil<0){
   Mil+=1000;
   Sec--;
  }
  Sec-=V[0]->Time.Second;
  T[i]=Sec + (double)Mil/1000;
 }

 //Compute the target time
 int Mil=Target.Millisecond;
 int Sec=Target.Second;
 Mil-=V[0]->Time.Millisecond;
 if(Mil<0){
  Mil+=1000;
  Sec--;
 }
 Sec-=V[0]->Time.Second;
 double TargetT=Sec + (double)Mil/1000;


 //XA=Y
 arma::mat X=arma::mat(nPoints,nPoints+1);
 arma::mat Y=arma::mat(nPoints,Size);

 //Fill the X matrix
 for(int i=0;i<nPoints;i++){
  for(int j=0;j<nPoints+1;j++){
   X(i,j)=pow(T[i],nPoints-j);
  }
 }

 //Fill the Y matrix
 for(int i=0;i<nPoints;i++)
  for(int j=0;j<Size;j++)
   Y(i,j)=V[i]->GetDoubleElement(j);

// X.print();
// Y.print();


 arma::mat A=solve(X,Y);

 arma::mat TargetX=arma::mat(1,nPoints+1);
 for(int i=0;i<nPoints+1;i++)TargetX(0,i)=pow(TargetT,nPoints-i);



 arma::mat Val=TargetX*A;



 Data.SetDataType(V[0]->DataType);
 Data.SetDimensions(V[0]->Dimensionality,V[0]->Dimensions);

 for(int i=0;i<V[0]->TotalNumberOfElements;i++)Data.SetDoubleElement(Val(0,i),i);




}
