#include "SADB.hpp"
#include "Interpolation/Linear.cpp"

#ifdef ARMADILLO
#include "Interpolation/Polynomial.cpp"
#endif

/// Constructor, initializes object
/// @see Blackboard::Initialize
Interpolator::Interpolator(void){Initialize();}

/// Destructor, deinitializes object
/// @see Blackboard::Free
Interpolator::~Interpolator(void){Free();}

/// Initializes object
void Interpolator::Initialize(void){
 Data.Initialize();
}

/// Frees allocated memory
void Interpolator::Free(void){
 Data.Free();
}

/// Set the target object
/// @param O
///  Target Object
void Interpolator::SetObject(SADBObject * O){
 Object=O;
}

/// Get the value of the object at a given timestamp
/// @param Target
///  Target time
/// @param Interpolation
///  Desired Interpolation Method
/// @param nPoints
///  Number of Points To Use
void Interpolator::GetValue(Timestamp Target,int Interpolation,int nPoints){
 pthread_mutex_lock(&Object->DataAllocationMutex);

 //printf("Received %d %d\n",Interpolation,nPoints);

 //Find the appropriate Value index (index of the previous value)
 Index=-1;
 for(int i=Object->nValues-1;i>=0;i--){
  int D=TimestampDifference(Object->Values[i]->Time,Target);
  if(D>0){Index=i;break;}
 }

 //Set timestamp of return value to the requested timestamp
 Data.Time.Second=Target.Second;
 Data.Time.Millisecond=Target.Millisecond;

#ifdef ARMADILLO
 if(Interpolation==PolynomialInterpolation)Polynomial(Target,nPoints);
 else 
#endif
  Linear(Target);//Defaults to linear

 pthread_mutex_unlock(&Object->DataAllocationMutex);
}
