#include "SADB.hpp"

Network::Network(void){}
Network::~Network(void){Close();}

/// Close a Socket
void Network::Close(void){
#ifdef POSIX
 close(Socket);
#endif
#ifdef WINDOWS
 closesocket(Socket);
#endif
}

/// Open a server socket
/// @param[in] port Port to use
void Network::Server(int port,int MaxQueue){
 Port=port;

 struct addrinfo Requested;
 memset(&Requested,0x00,sizeof(Requested));
 Requested.ai_family=AF_UNSPEC;//IPv4 or IPv6
 Requested.ai_socktype=SOCK_STREAM;
 Requested.ai_flags=AI_PASSIVE;//Auto Determine IP

 char * PortStr=(char *)malloc(10);
 sprintf(PortStr,"%d",Port);

 if(getaddrinfo(NULL,PortStr,&Requested,&AddressInfo)!=0)Error("Network","Cannot Get Address Information");
 if((Socket=socket(AddressInfo->ai_family,AddressInfo->ai_socktype,AddressInfo->ai_protocol))==-1)Error("Network","Cannot Open Socket");
 if(bind(Socket,AddressInfo->ai_addr,AddressInfo->ai_addrlen)==-1)Error("Network","Cannot Bind To Socket");
 if(listen(Socket,MaxQueue)==-1)Error("Network","Cannot Listen on Socket");
 free(PortStr);
 
 //Set Nodelay
 int flag=1;
 setsockopt(Socket,IPPROTO_TCP,TCP_NODELAY,&flag,sizeof(flag));
 
 //SetNonBlocking
 //int CurrentFlags=fcntl(Socket,F_GETFL,0);
 //if(CurrentFlags<0)Error("Network","Cannot Read current Flags");
 //CurrentFlags|=O_NONBLOCK;//Non-blocking
 //fcntl(Socket,F_SETFL,CurrentFlags);
}

///Get a queued connection (if any)
///@return Socket Descriptor
int Network::GetConnection(void){
 struct sockaddr_storage NewAddress;
 socklen_t AddressSize=sizeof(NewAddress);
 int NewSocketFD=accept(Socket,(struct sockaddr *)&NewAddress,&AddressSize);
 int flag=1;
 if(NewSocketFD>=0)setsockopt(this->Socket,SOL_TCP,TCP_NODELAY,&flag,sizeof(flag));//Set no delay on new socket
 return NewSocketFD;
}

///Wait for a new connection
///@return Socket Descriptor
int Network::WaitForConnection(void){
 int Socket=GetConnection();
 while(Socket<0)Socket=GetConnection();//Loop until a valid socket is obtained
 return Socket;
}

/// Connect to a server
/// @param[in] address Address to connect to
/// @param[in] port Port to use
void Network::ConnectTo(const char * address,int port){
 Port=port;

 struct addrinfo Requested;
 memset(&Requested,0x00,sizeof(Requested));
 Requested.ai_family=AF_UNSPEC;//IPv4 or IPv6
 Requested.ai_socktype=SOCK_STREAM;

 char * PortStr=(char *)malloc(10);
 sprintf(PortStr,"%d",Port);

 if(getaddrinfo(address,PortStr,&Requested,&AddressInfo)!=0)Error("Network","Cannot Get Address Information");
 if((Socket=socket(AddressInfo->ai_family,AddressInfo->ai_socktype,AddressInfo->ai_protocol))==-1)Error("Network","Cannot Open Socket");
 if(connect(Socket,AddressInfo->ai_addr,AddressInfo->ai_addrlen)==-1)Error("Network","Cannot Connect to Socket");

 int flag=1;
 setsockopt(Socket,IPPROTO_TCP,TCP_NODELAY,&flag,sizeof(flag));

 setsockopt(Socket,SOL_TCP,TCP_CORK,&flag,sizeof(flag));
}













/// Receive data from the socket
/// @param[in] Socket Socket to use
/// @param[out] Buffer Pointer to memory location where data read is to be stored
/// @param[in] MaxLength Maximum data to be read
/// @return Number of bytes read
int Network::Receive(unsigned char * Buffer,int MaxLength){
 return SocketReceive(Socket,Buffer,MaxLength);
}

/// Receive a complete Packet from the socket
/// @param[in] Socket Socket to use
/// @param[out] Buffer Pointer to memory location where data read is to be stored
/// @param[in] Length Maximum data to be read
/// @return Number of bytes read
int Network::ReceiveExactly(unsigned char * Buffer,int Length){
 return SocketReceiveExactly(Socket,Buffer,Length);
} 

/// Send a string
/// @param[in] Socket Socket to use
/// @param[out] Buffer Pointer to memory location where data to be sent is stored
void Network::Send(unsigned char * Buffer){
 SocketSend(Socket,(const char*)Buffer);
}

/// Send Data
/// @param[in] Socket Socket to use
/// @param[in] Buffer Pointer to memory location where data to be sent is stored
/// @param[in] Length Number of bytes to be sent
void Network::Send(unsigned char * Buffer,int Length){
 SocketSend(Socket,Buffer,Length);
}

/// Send a Formatted String
/// @param[in] Socket Socket to use
/// @param[out] format Format string
/// @param[in] ... Additional variables
void Network::Sendf(unsigned char *format,...){
 va_list args;
 va_start(args,format);
 char*Buffer;
 vasprintf(&Buffer,(const char*)format,args);
 SocketSend(Socket,(const char*)Buffer);
}


















/// Receive data from the socket
/// @param[in] Socket Socket to use
/// @param[out] Buffer Pointer to memory location where data read is to be stored
/// @param[in] MaxLength Maximum data to be read
/// @return Number of bytes read
int SocketReceive(int Socket,unsigned char * Buffer,int MaxLength){
 int length=recv(Socket,(char*)Buffer,MaxLength,0);
 return length;
}

/// Receive a complete Packet from the socket
/// @param[in] Socket Socket to use
/// @param[out] Buffer Pointer to memory location where data read is to be stored
/// @param[in] Length Maximum data to be read
/// @return Number of bytes read
int SocketReceiveExactly(int Socket,unsigned char * Buffer,int Length){
 //Block until a complete packet has been received
 int Received=0;
 while(Received<Length){
  int l=recv(Socket,(char*)Buffer,Length-Received,0);
  Received+=l;
  Buffer+=l;
  if(l==0)return l;
 }
 return Received;
}


/// Send Data
/// @param[in] Socket Socket to use
/// @param[in] Buffer Pointer to memory location where data to be sent is stored
/// @param[in] Length Number of bytes to be sent
int SocketSend(int Socket,unsigned char * Buffer,long int Length){
 long int Sent=send(Socket,(const char *)Buffer,Length,0);
 while(Sent<Length){
  int s=send(Socket,(const char *)Buffer,Length-Sent,0);
  if(s==-1)return -1;
  Sent+=s;
  Buffer+=s;
 }
 return 0;
}

///
/// Send a string
/// @param[in] Socket
///   Socket to use
/// @param[out] Buffer
///   Pointer to memory location where data to be sent is stored
///
int SocketSend(int Socket,const char * Buffer){
 return SocketSend(Socket,(unsigned char*)Buffer,strlen((const char*)Buffer));
}


/// Send a Formatted String
/// @param[in] Socket Socket to use
/// @param[out] format Format string
/// @param[in] ... Additional variables
int SocketSendf(int Socket,const char *format,...){
 va_list args;
 va_start(args,format);
 char*Buffer;
 vasprintf(&Buffer,(const char*)format,args);
 return SocketSend(Socket,(const char*)Buffer);
}


void CorkSocket(int Socket){
 int flag=1;setsockopt(Socket,SOL_TCP,TCP_CORK,&flag,sizeof(flag));
}
void UncorkSocket(int Socket){
 int flag=0;setsockopt(Socket,SOL_TCP,TCP_CORK,&flag,sizeof(flag));
}
