#include "SADB.hpp"

/// Constructor, initializes object
/// @see SADBRequest::Initialize
SADBRequest::SADBRequest(void){Initialize();}

/// Destructor, deinitializes object
/// @see SADBRequest::Free
SADBRequest::~SADBRequest(void){Free();}

/// Initializes object
void SADBRequest::Initialize(void){
 IDs=NULL;nIDs=0;
}

/// Frees allocated memory
void SADBRequest::Free(void){
 if(IDs!=NULL)free(IDs);
}

void SADBRequest::SetBoard(SADBServer*board){
 Board=board;
}


int SADBRequest::Valid(char *S){//Check to make sure all delimiters are there
 int L=strlen(S);
 int a=0;//Opening
 int b=0;//Closing
 for(int i=0;i<L;i++){
  if(S[i]=='(')a++;
  else if(S[i]==')')b++;
 }
 if(a!=b)return 0;
 else return 1;
}



void SADBRequest::Process(u8*S,u64**List,int*nObjects){
 *nObjects=0;//To be safe


 //Strip the trailing and leading whitespace
 int SLength=strlen((const char *)S);
 int ls=0,ts=0;
 for(int i=0;i<SLength;i++){if((S[i]==' ')||(S[i]=='\t'))ls++;else break;}
 for(int i=SLength-1;i>0;i--){if((S[i]==' ')||(S[i]=='\t'))ts++;else break;}
 //printf("\"%s\" = ls=%d ts=%d sl=%d\n",(const char*)S,ls,ts,SLength);//Debug Print
 char * Truncated=(char *)malloc((SLength-ls-ts+1)*sizeof(char));//+1 for the '\0'
 for(int i=ls;i<=(SLength-ts);i++){
  Truncated[i-ls]=S[i];
 }Truncated[SLength-ls-ts]='\0';

 //Allocate memory to hold the different parts of the string
 SLength=strlen(Truncated)+1;//max possible length,+1 for the '\0'
 char * CMD=(char*)malloc(SLength*sizeof(char));
 char * Inside=(char*)malloc(SLength*sizeof(char));

/*
 char * Start=&Truncated[0];
 char * Del=strchr(Start,'(');
 if(Del!=NULL){//Function
 memcpy(CMD,Start,(Del-Start)*sizeof(char));CMD[Del-Start]='\0';
 memcpy(Inside,Del,SLength-(Del-Start)*sizeof(char));Inside[SLength-(Del-Start)]='\0';
 printf("%s - %s\n",CMD,Inside);
*/


 if(sscanf(Truncated,"%[^(](%[ ().,A-Za-z0-9]",CMD,Inside)==2){//Function


  int L=strlen(Inside);
  Inside[L-1]='\0';
  L--;
 
    //printf("%s - %s\n",CMD,Inside);

  //Count commas to find number of arguments
  int nArguments=1;
  int inSub=0;
  for(int i=0;i<L-1;i++){
   if(Inside[i]=='(')inSub++;
   if(Inside[i]==')')inSub--;
   if((Inside[i]==',')&&(inSub==0))nArguments++;
  }

  //Allocate memory to hold the arguments
  char ** Arguments;
  Arguments=(char **)malloc(nArguments*sizeof(char*));
  for(int i=0;i<nArguments;i++)Arguments[i]=(char *)malloc(L*sizeof(char));//Allocating the max possible amount of memory

  //Extract the arguments
  int cArgument=0;//Current Argument
  int cPosition=0;//Current Position
  inSub=0;
  for(int i=0;i<=L;i++){
   if(Inside[i]=='(')inSub++;
   if(Inside[i]==')')inSub--;

   if((Inside[i]==',')&&(inSub==0)){
    Arguments[cArgument][cPosition]='\0';
    cPosition=0;
    cArgument++;
   }else{
    Arguments[cArgument][cPosition]=Inside[i];
    cPosition++;
   }
  }

  //Now, reprocess every argument to get their list of objects
  //Could reduce memory requirement here by only fetching one list at a time, but we've got plenty of ram
  u64 ** ListOfObjects=(u64**)malloc(nArguments*sizeof(u64*));
  int * ListOfObjectsLength=(int*)malloc(nArguments*sizeof(int));
  for(int i=0;i<nArguments;i++){
   //printf("Processing: %s\n",Arguments[i]);
   Process((unsigned char*)Arguments[i],&ListOfObjects[i],&ListOfObjectsLength[i]);
  }



  for(u64 i=0;i<strlen(CMD);i++)CMD[i]=(char)toupper((int)CMD[i]);//make string uppercase


  if(((strcmp(CMD,"UN")==0)||(strcmp(CMD,"UNION")==0))&&(nArguments>1)){
   u64 * IntList;int nIntList;
   u64 * TMPList;int nTMPList;
   Union(ListOfObjects[0],(int)ListOfObjectsLength[0],ListOfObjects[1],(int)ListOfObjectsLength[1],&TMPList,&nTMPList);
   if(nArguments>2){//if more than 2 arguments
    for(int i=2;i<nArguments;i++){
     IntList=TMPList;nIntList=nTMPList;
     TMPList=NULL;
     Union(IntList,nIntList,ListOfObjects[i],ListOfObjectsLength[i],&TMPList,&nTMPList);
     free(IntList);
    }
   }
   *nObjects=nTMPList;
   *List=TMPList;
  }
  if(((strcmp(CMD,"IN")==0)||(strcmp(CMD,"INTERSECTION")==0))&&(nArguments>1)){
   u64 * IntList;int nIntList;
   u64 * TMPList;int nTMPList;
   Intersection(ListOfObjects[0],(int)ListOfObjectsLength[0],ListOfObjects[1],(int)ListOfObjectsLength[1],&TMPList,&nTMPList);
   if(nArguments>2){//if more than 2 arguments
    for(int i=2;i<nArguments;i++){
     IntList=TMPList;nIntList=nTMPList;
     TMPList=NULL;
     Intersection(IntList,nIntList,ListOfObjects[i],ListOfObjectsLength[i],&TMPList,&nTMPList);
     free(IntList);
    }
   }
   *nObjects=nTMPList;
   *List=TMPList;
   //printf("Intersection is :\n");
   //for(int r=0;r<nTMPList;r++)printf("  %lx\n",TMPList[r]);
  }
  if(((strcmp(CMD,"CO")==0)||(strcmp(CMD,"COMPLEMENT")==0))&&(nArguments==1)){
   Complement(ListOfObjects[0],ListOfObjectsLength[0],List,nObjects);
  }

 
  for(int i=0;i<nArguments;i++){
   free(Arguments[i]);
   if(ListOfObjectsLength[i]>0)free(ListOfObjects[i]);
  }
  free(Arguments);
  free(ListOfObjects);
  free(ListOfObjectsLength);


 }else{//Is not a command, thus it's a category
  u64 CUID=Board->GetCategoryByName((u8*)Truncated);
  //printf(" - %s (%lx)\n",Truncated,CUID);
  if(Board->IsValidCUID(CUID)){//If category is valid, we return the list of its elements
   SADBCategory * Category=Board->Categories[CUID-1];
   *nObjects=Category->nObjects;
   *List=(u64 *)malloc(Category->nObjects*sizeof(u64));//Don't forget to dealloc this
   memcpy(*List,Category->ObjectIDs,Category->nObjects*sizeof(u64));
   for(int i=0;i<Category->nObjects;i++){
    //printf("    %lx\n",Category->ObjectIDs[i]);
   }
  }
 }

 if(Inside!=NULL)free(Inside);
 if(Inside!=NULL)free(Truncated);
 if(Inside!=NULL)free(CMD);


}



/// Get the union of 2 lists
/// @param ListA
///  First List
/// @param nElementsA
///  Number of elements in the first list
/// @param ListB
///  Second List
/// @param nElementsB
///  Number of elements in the second list
/// @param Output
///  Output List
/// @param nElementsOutput
///  Number of elements in the output list
void SADBRequest::Union(u64 * ListA,int nElementsA,u64 * ListB,int nElementsB,u64 ** Output,int * nElementsOutput){
 //Malloc enough memory
 *Output=(u64 *)malloc((nElementsA+nElementsB)*sizeof(u64));
 if(Output==NULL){
  *nElementsOutput=0;
  return;
 }

 int iO=0;//Index in output list

 //Really inneficient method, but it works so for now it stays

 for(int i=0;i<nElementsA;i++){//Put all elements of A into Out
  (*Output)[iO]=ListA[i];
  iO++;
 }



 for(int i=0;i<nElementsB;i++){//Put all elements of B into Out
  int Position=0;

  for(int j=0;j<iO;j++){
   if(ListB[i]<(*Output)[j]){Position=j;break;}
   else Position=j+1;
  }
  if(!((Position>0)&&(ListB[i]==(*Output)[Position-1]))){//object is not already in list, skip it
  
   if(Position<iO){//Need to shift list
    for(int j=iO;j>=Position;j--)(*Output)[j]=(*Output)[j-1];
   }
   (*Output)[Position]=ListB[i];
   iO++;
  }
 }

 *Output=(u64*)realloc(*Output,iO*sizeof(u64));//Shrink list to right size
 if(*Output==NULL)*nElementsOutput=0;
 else *nElementsOutput=iO;


}

/// Get the intersection of 2 lists
/// @param ListA
///  First List
/// @param nElementsA
///  Number of elements in the first list
/// @param ListB
///  Second List
/// @param nElementsB
///  Number of elements in the second list
/// @param Output
///  Output List
/// @param nElementsOutput
///  Number of elements in the output list
void SADBRequest::Intersection(u64 * ListA,int nElementsA,u64 * ListB,int nElementsB,u64 ** Output,int * nElementsOutput){
 //printf("A\n");for(int i=0;i<nElementsA;i++)printf(" %ld\n",ListA[i]);
 //printf("B\n");for(int i=0;i<nElementsB;i++)printf(" %ld\n",ListB[i]);
 

 int MaxLength=nElementsA;
 if(nElementsB>nElementsA)MaxLength=nElementsB;
 //Malloc enough memory
 *Output=(u64 *)malloc(MaxLength*sizeof(u64));
 int iO=0;//Index in output list

 for(int i=0;i<nElementsA;i++){//Go through list A, would probably be more efficient to check for the smaller list and use that one
  int Match=0;
  for(int j=0;j<nElementsB;j++){//Compare to objects in list B
   if(ListB[j]>ListA[i])break;//Lists are sorted, so we can stop once a larger ID is seen in ListB
   if(ListB[j]==ListA[i]){Match=1;break;}//A Match is found
  }
  if(Match==1){
   (*Output)[iO]=ListA[i];//Add to output list
   iO++;
  }

 }

 *Output=(u64*)realloc(*Output,iO*sizeof(u64));//Shrink list to right size
 *nElementsOutput=iO;
}



/// Get the complement of a lists
/// @param ListA
///  First List
/// @param nElementsA
///  Number of elements in the first list
/// @param Output
///  Output List
/// @param nElementsOutput
///  Number of elements in the output list
void SADBRequest::Complement(u64 * ListA,int nElementsA,u64 ** Output,int * nElementsOutput){
 //Malloc enough memory
 *Output=(u64 *)malloc(Board->nObjects*sizeof(u64));
 int iA=0;//Index in list A
 int iO=0;//Index in output list

 for(u64 i=0;i<(u64)Board->nObjects;i++){
  if(i!=ListA[iA]){*Output[iO]=i;iO++;}
  else{iA++;if(iA>=nElementsA)iA=nElementsA-1;}
 }

 *Output=(u64*)realloc(Output,iO*sizeof(u64));//Shrink list to right size
 *nElementsOutput=iO;
}
