#include "SADB.hpp"

SADBServer::SADBServer(void){Initialize();}
SADBServer::~SADBServer(void){Free();}

void SADBServer::Initialize(void){
 //Objects initialization
 pthread_mutex_init(&ObjectAllocationMutex,NULL);
 CurrentObjectID=1;//Start at 1 because 0 is reserved
 Objects=NULL;nObjects=0;ObjectStates=NULL;
 //Categories initialization
 pthread_mutex_init(&CategoryAllocationMutex,NULL);
 CurrentCategoryID=1;//Start at 1 because 0 is reserved
 Categories=NULL;nCategories=0;CategoryStates=NULL;
}


void SADBServer::Free(void){
 //Objects
 pthread_mutex_lock(&ObjectAllocationMutex);
 if(Objects!=NULL)free(Objects);nObjects=0;Objects=NULL;
 if(ObjectStates!=NULL)free(ObjectStates);ObjectStates=NULL;
 pthread_mutex_unlock(&ObjectAllocationMutex);
 pthread_mutex_destroy(&ObjectAllocationMutex);
 //Categories
 pthread_mutex_lock(&CategoryAllocationMutex);
 if(Categories!=NULL)free(Categories);nCategories=0;Categories=NULL;
 if(CategoryStates!=NULL)free(CategoryStates);CategoryStates=NULL;
 pthread_mutex_unlock(&CategoryAllocationMutex);
 pthread_mutex_destroy(&CategoryAllocationMutex);
}


void SADBServer::Reset(void){
 //Delete objects
 pthread_mutex_lock(&ObjectAllocationMutex);
  for(u64 i=0;i<nObjects;i++){
   if(ObjectStates[i]){
    Objects[i]->Free();
    free(Objects[i]);
   }
  }
  nObjects=0;
  free(Objects);Objects=NULL;
  free(ObjectStates);ObjectStates=NULL;
  CurrentObjectID=1;
 pthread_mutex_unlock(&ObjectAllocationMutex);
 //Delete categories
 pthread_mutex_lock(&CategoryAllocationMutex);
  for(u64 i=0;i<nCategories;i++){
   CategoryStates[i]=0;
   Categories[i]->Free();//Free memory allocated by object
   free(Categories[i]);Categories[i]=NULL; 
  }
  nCategories=0;
  free(Categories);Categories=NULL;
  free(CategoryStates);CategoryStates=NULL;
  CurrentCategoryID=1;
 pthread_mutex_unlock(&CategoryAllocationMutex);
}



void *SADBServer::SADBClientHandler(void*ClientInfo){
 SADBClientInfo*Info=(SADBClientInfo*)ClientInfo;
 printf("Client Thread Started on socket %d\n",Info->Socket);
 u8 Command=0;
 u64 OUID,Argument;
 Timestamp Time;
 u32 DataLength;
 u8*Data=NULL;

 while(1){//Once connected, client can execute multiple commands
  int r=SADBReceive2(Info->Socket,&Command,&OUID,&Argument,&(Time.Second),&(Time.Millisecond),&DataLength,&Data);
  //printf("Received Command %d\n",Command);
  if(r==1)break;//Disconected, stop thread
  if(r==0){//Valid Command
   switch(Command){
    case 0x01:Info->Board->CMDPing(Info);break;
    case 0x02:Info->Board->CMDCreateObject(Info);break;
    case 0x03:Info->Board->CMDDeleteObject(Info,OUID);break;
    case 0x04:Info->Board->CMDSetObjectName(Info,OUID,Data);break;
    case 0x05:Info->Board->CMDGetObjectName(Info,OUID);break;
    case 0x06:Info->Board->CMDSetObjectDescription(Info,OUID,Data);break;
    case 0x07:Info->Board->CMDGetObjectDescription(Info,OUID);break;
    case 0x08:Info->Board->CMDGetObjectByName(Info,Data);break;
    case 0x09:Info->Board->CMDSendObject(Info,OUID,Argument,Time.Second,Time.Millisecond,Data);break;
    case 0x0A:Info->Board->CMDGetLatestObjectValue(Info,OUID);break;
    case 0x0B:Info->Board->CMDGetObjectValueAtNearestTimestamp(Info,OUID,Time);break;
    case 0x0C:Info->Board->CMDGetObjectValueAtTimestamp(Info,OUID,Time,Argument);break;
    case 0x0D:Info->Board->CMDReset(Info);break;
    case 0x0E:Info->Board->CMDCreateCategory(Info);break;
    case 0x0F:Info->Board->CMDDeleteCategory(Info,OUID);break;
    case 0x10:Info->Board->CMDSetCategoryName(Info,OUID,Data);break;
    case 0x11:Info->Board->CMDGetCategoryName(Info,OUID);break;
    case 0x12:Info->Board->CMDSetCategoryDescription(Info,OUID,Data);break;
    case 0x13:Info->Board->CMDGetCategoryDescription(Info,OUID);break;
    case 0x14:Info->Board->CMDGetCategoryByName(Info,Data);break;
    case 0x15:Info->Board->CMDAddObjectToCategory(Info,OUID,Argument);break;
    case 0x16:Info->Board->CMDRemoveObjectFromCategory(Info,OUID,Argument);break;
    case 0x17:Info->Board->CMDFindObjects(Info,Data);break;
    case 0x18:Info->Board->CMDRemoveOldestValue(Info,OUID);break;
    case 0x19:Info->Board->CMDKeepOnlyLatestValues(Info,OUID,Argument);break;
    case 0x1A:Info->Board->CMDGetLatestObjectTimestamp(Info,OUID);break;
    case 0x1B:Info->Board->CMDGetObjectValueAtNextTimestamp(Info,OUID,Time);break;
    case 0x1C:Info->Board->CMDGetObjectValueAtPreviousTimestamp(Info,OUID,Time);break;
    case 0x1D:Info->Board->CMDGetNextObjectID(Info);break;
    case 0x1E:Info->Board->CMDGetNextCategoryID(Info);break;
    case 0x20:Info->Board->CMDIsValidObject(Info,OUID);break;
    case 0x21:Info->Board->CMDIsValidCategory(Info,OUID);break;
    case 0x22:Info->Board->CMDGetNextObjectTimestamp(Info,OUID,Time);break;
    case 0x23:Info->Board->CMDGetPreviousObjectTimestamp(Info,OUID,Time);break;
    case 0x24:Info->Board->CMDGetNearestObjectTimestamp(Info,OUID,Time);break;
    default:;
   } 
  }
 }
 //Shut down
 close(Info->Socket);
 printf("Client Thread Stopped on socket %d\n",Info->Socket);
 free(Info);//Prevent Memory Leaks
 pthread_exit(NULL);
}









void SADBServer::Start(int port){
 StopRequest=0;//No Stop Request
 Port=port;
 if(pthread_create(&ServerMainThreadID,NULL,StartMainThreadProxy,this))Error("SADB Server","Cannot start thread");
 pthread_detach(ServerMainThreadID);
}

void SADBServer::Stop(void){
 StopRequest=1;
}

void *SADBServer::StartMainThreadProxy(void *context){
 ((SADBServer *)context)->Main();//Call server main
 return NULL;

}

void SADBServer::Main(void){
 Server.Server(Port,5);
 printf("SADB Server Started on port %d\n",Port);
 while(1){//Check for new connections
  int Socket=Server.GetConnection();//Get Connection
  if(Socket>=0){//Valid Socket
   SADBClientInfo*Client=(SADBClientInfo*)malloc(sizeof(SADBClientInfo));//Allocate memory, let  the client handler free it
   Client->Socket=Socket;//the socket FD
   Client->Board=this;//Pointer to the server that spawned the client handler
   //Start a new thread to handle the request and detatch it
   pthread_t RequestHandler;
   if(pthread_create(&RequestHandler,NULL,SADBClientHandler,Client))Error("SADB","Failed to spawn client handler thread");
   pthread_detach(RequestHandler);
  }
  if(StopRequest==1){
   printf("SADB Server Stopped\n");
   pthread_exit(NULL);
  }
 }
}




//Server Object Related functions

int SADBServer::IsValidOUID(u64 OUID){
 int Valid=1;
 if(OUID<1)return 0;
 if(OUID>(u64)nObjects)return 0;
 pthread_mutex_lock(&ObjectAllocationMutex);
 if(ObjectStates[OUID-1]!=1)Valid=0;
 pthread_mutex_unlock(&ObjectAllocationMutex);
 return Valid;
}

u64 SADBServer::CreateObject(void){
 pthread_mutex_lock(&ObjectAllocationMutex);
  u64 ID=CurrentObjectID;CurrentObjectID++;//Increase ID for next created object
  nObjects++;//Increase number of objects
  Objects=(SADBObject **)realloc(Objects,nObjects*sizeof(SADBObject*));//Allocate more memory
  if(Objects==NULL)Error("Memory","Cannot allocate object memory");
  ObjectStates=(u8 *)realloc(ObjectStates,nObjects*sizeof(u8));//Allocate more memory
  if(ObjectStates==NULL)Error("Memory","Cannot allocate object state memory");
  Objects[nObjects-1]=(SADBObject *)malloc(sizeof(SADBObject));
  Objects[nObjects-1]->Initialize();
  Objects[nObjects-1]->ID=ID;//Set the ID of the new object
  ObjectStates[nObjects-1]=1;
 pthread_mutex_unlock(&ObjectAllocationMutex);
 return ID;
}

int SADBServer::RemoveObject(u64 ID){
 if(IsValidOUID(ID)){
  pthread_mutex_lock(&ObjectAllocationMutex);
  ObjectStates[ID-1]=0;//Not active anymore
  Objects[ID-1]->Free();//Free memory allocated by object
  free(Objects[ID-1]);Objects[ID-1]=NULL;
  pthread_mutex_unlock(&ObjectAllocationMutex);
  return 0;
 }else return 1;
}


//Server Category related functions

int SADBServer::IsValidCUID(u64 CUID){
 int Valid=1;
 if(CUID<1)return 0;
 if(CUID>(u64)nCategories)return 0;
 pthread_mutex_lock(&CategoryAllocationMutex);
 if(CategoryStates[CUID-1]!=1)Valid=0;
 pthread_mutex_unlock(&CategoryAllocationMutex);
 return Valid;
}



u64 SADBServer::CreateCategory(void){
 pthread_mutex_lock(&CategoryAllocationMutex);
  u64 ID=CurrentCategoryID;CurrentCategoryID++;//Increase ID for next created category
  nCategories++;//Increase number of categories
  Categories=(SADBCategory **)realloc(Categories,nCategories*sizeof(SADBCategory*));//Allocate more memory
  if(Categories==NULL)Error("Memory","Cannot allocate category memory");
  CategoryStates=(u8 *)realloc(CategoryStates,nCategories*sizeof(u8));//Allocate more memory
  if(CategoryStates==NULL)Error("Memory","Cannot allocate category memory");
  Categories[nCategories-1]=(SADBCategory *)malloc(sizeof(SADBCategory));
  Categories[nCategories-1]->Initialize();
  Categories[nCategories-1]->ID=ID;//Set the ID of the new category
  CategoryStates[nCategories-1]=1;
 pthread_mutex_unlock(&CategoryAllocationMutex);
 return ID;
}

int SADBServer::RemoveCategory(u64 ID){
 if(IsValidCUID(ID)){
  pthread_mutex_lock(&CategoryAllocationMutex);
  CategoryStates[ID-1]=0;
  Categories[ID-1]->Free();//Free memory allocated by object
  free(Categories[ID-1]);Categories[ID-1]=NULL; 
  pthread_mutex_unlock(&CategoryAllocationMutex);
  return 0;
 }else return 1;
}

u64 SADBServer::GetCategoryByName(u8*Name){
 //printf("Looking for %s\n",Name);
 u64 ID=0;
 for(u64 i=0;i<nCategories;i++){
  if(IsValidCUID(i+1)){
   pthread_mutex_lock(&(Categories[i]->InfoMutex));//Make sure no one is writing names when we read them
   if((Categories[i]->Name)!=NULL){//If a name has been set
    if(strcmp( (const char*)(Categories[i]->Name) , (const char *)Name ) == 0 ){//identical
     ID=Categories[i]->ID;
     pthread_mutex_unlock(&(Categories[i]->InfoMutex));//Make sure to unlock before breaking
     break;
    }
   }
   pthread_mutex_unlock(&(Categories[i]->InfoMutex));
  }
 }
 //printf("Found %d\n",ID);
 return ID;
}










//Server Commands

void SADBServer::CMDPing(SADBClientInfo*Info){
 Timestamp Pong;GetTimestamp(Pong);
 SendSADBPacket(Info->Socket,0x00,0,0,Pong.Second,Pong.Millisecond,0,NULL);//Send a response
}

void SADBServer::CMDReset(SADBClientInfo*Info){
 Info->Board->Reset();
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,NULL);//Send a response
}

void SADBServer::CMDFindObjects(SADBClientInfo*Info,u8*String){
 u64*List=NULL;int nListElements;
 SADBRequest Request;Request.SetBoard(Info->Board);
 Request.Process(String,&List,&nListElements);
 //Send Response
 if(nListElements==0)SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//No match
 else if(nListElements==1)SendSADBPacket(Info->Socket,0x00,List[0],0,0,0,0,0);//Single Match
 else SendSADBPacket(Info->Socket,0x00,0,0,0,0,nListElements*sizeof(u64),(u8*)List);//Multiple Matches
 if(nListElements!=0)free(List);
}

void SADBServer::CMDCreateObject(SADBClientInfo*Info){
 u64 ID=Info->Board->CreateObject();
 SendSADBPacket(Info->Socket,0x00,ID,0,0,0,0,NULL);//Send a response
}

void SADBServer::CMDDeleteObject(SADBClientInfo*Info,u64 OUID){
 Info->Board->RemoveObject(OUID);
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,NULL);//Send a response
}

void SADBServer::CMDIsValidObject(SADBClientInfo*Info,u64 OUID){
 if(Info->Board->IsValidOUID(OUID))SendSADBPacket(Info->Socket,0x00,0,1,0,0,0,NULL);
 else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,NULL);
}

void SADBServer::CMDSetObjectName(SADBClientInfo*Info,u64 OUID,u8 * Name){
 if(Info->Board->IsValidOUID(OUID))(Info->Board->Objects[OUID-1])->SetName((const char *)Name);//Set the name
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,NULL);//Send a response
}

void SADBServer::CMDGetObjectName(SADBClientInfo*Info,u64 OUID){
 if(Info->Board->IsValidOUID(OUID)){
  pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->InfoMutex));
  if((Info->Board->Objects[OUID-1])->Name==NULL){
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,1,(u8*)"");//Send a response
  }else{
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,
    strlen((const char*)((Info->Board->Objects[OUID-1])->Name))+1,
    ((Info->Board->Objects[OUID-1])->Name) );//Send a response
  }
  pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->InfoMutex));
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,1,(u8*)"");//Send a response
}

void SADBServer::CMDSetObjectDescription(SADBClientInfo*Info,u64 OUID,u8 * Description){
 if(Info->Board->IsValidOUID(OUID))(Info->Board->Objects[OUID-1])->SetDescription((const char *)Description);//Set the description
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,NULL);//Send a response
}

void SADBServer::CMDGetObjectDescription(SADBClientInfo*Info,u64 OUID){
 if(Info->Board->IsValidOUID(OUID)){
  pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->InfoMutex));
  if((Info->Board->Objects[OUID-1])->Description==NULL){
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,1,(u8*)"");//Send a response
  }else{
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,
    strlen((const char*)((Info->Board->Objects[OUID-1])->Description))+1,
    ((Info->Board->Objects[OUID-1])->Description));//Send a response
  }
  pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->InfoMutex));
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,1,(u8*)"");//Send a response
}

void SADBServer::CMDGetObjectByName(SADBClientInfo*Info,u8*Name){
 //printf("Looking for: %s\n",Name);
 u64 ID=0;
 for(u64 i=1;i<=Info->Board->nObjects;i++){
  if(Info->Board->IsValidOUID(i)){
   pthread_mutex_lock(&((Info->Board->Objects[i-1])->InfoMutex));//Make sure no one is writing names when we read them
   if(((Info->Board->Objects[i-1])->Name)!=NULL){//If a name has been set
    //printf(" comparing with: %s\n",(const char*)((Info->Board->Objects[i-1])->Name));
    if(strcmp( (const char*)((Info->Board->Objects[i-1])->Name) , (const char *)Name ) == 0 ){//identical
     //printf("%s = %s\n",((Info->Board->Objects[i-1])->Name),Name);
     ID=(Info->Board->Objects[i-1])->ID;
     pthread_mutex_unlock(&((Info->Board->Objects[i-1])->InfoMutex));//Make sure to unlock before breaking
     break;
    }
   }
   pthread_mutex_unlock(&((Info->Board->Objects[i-1])->InfoMutex));
  }
 }
 //printf("Found %ld\n",ID);
 SendSADBPacket(Info->Socket,0x00,ID,0,0,0,0,0);//Send a response
}

void SADBServer::CMDSendObject(SADBClientInfo*Info,u64 OUID,u32 Datatype,u64 Second,u16 Millisecond,u8*Data){
 u32 D=((u32*)Data)[0];
 if(Info->Board->IsValidOUID(OUID))
  (Info->Board->Objects[OUID-1])->NewValue(D,(u32*)(Data+sizeof(u32)),Datatype,Data+(D+1)*sizeof(u32),Second,Millisecond);
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Send a response
 //((Info->Board->Objects[OUID-1])->Values[(Info->Board->Objects[OUID-1])->nValues-1])->Information();//print info for debugging
}

void SADBServer::CMDGetLatestObjectValue(SADBClientInfo*Info,u64 OUID){
 //Send Packet, Argument is the data type, the first 4 bytes of data is the number of dimensions
 //the next n*4 bytes are the dimensions, then the data
 if((Info->Board->IsValidOUID(OUID))&&((Info->Board->Objects[OUID-1])->nValues>0)){
  int ValueIndex=((Info->Board->Objects[OUID-1])->nValues)-1;//Lastest object value
  pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));//Make sure that the new value is not being allocated
  SADBObjectStorage * Object=(Info->Board->Objects[OUID-1])->Values[ValueIndex];
  pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
  if(pthread_mutex_trylock(&(Object->Mutex)) == 0){//Latest value is available
   u32 * Buffer=(u32*)malloc((Object->Dimensionality+1)*sizeof(u32));
   Buffer[0]=Object->Dimensionality;
   for(int i=0;i<Object->Dimensionality;i++)Buffer[i+1]=Object->Dimensions[i];
   SendSADBPacket(Info->Socket,0x00,OUID,Object->DataType,(Object->Time).Second,(Object->Time).Millisecond,
    (Object->Dimensionality+1)*sizeof(u32),(u8 *)Buffer,//Dimension information
    Object->TotalNumberOfElements*Object->ElementSize,(u8*)Object->Data);//Actual Data
   free(Buffer);
   pthread_mutex_unlock(&(Object->Mutex));
  }else if((Info->Board->Objects[OUID-1])->nValues==1){//Only one value, so we'll wait for the mutex...
   pthread_mutex_lock(&(Object->Mutex));
   u32 * Buffer=(u32*)malloc((Object->Dimensionality+1)*sizeof(u32));//max of 99 dimensions, can be changed
   Buffer[0]=Object->Dimensionality;
   for(int i=0;i<Object->Dimensionality;i++)Buffer[i+1]=Object->Dimensions[i];
   SendSADBPacket(Info->Socket,0x00,OUID,Object->DataType,(Object->Time).Second,(Object->Time).Millisecond,
    (Object->Dimensionality+1)*sizeof(u32),(u8 *)Buffer,//Dimension information
    Object->TotalNumberOfElements*Object->ElementSize,(u8*)Object->Data);//Actual Data
   free(Buffer);
   pthread_mutex_unlock(&(Object->Mutex));
  }else{//Latest value is currently being used, so we'll return the one before
   pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));//Make sure that the new value is not being allocated
   SADBObjectStorage * Object=(Info->Board->Objects[OUID-1])->Values[ValueIndex-1];
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   pthread_mutex_lock(&(Object->Mutex));
   u32 * Buffer=(u32*)malloc((Object->Dimensionality+1)*sizeof(u32));//max of 99 dimensions, can be changed
   Buffer[0]=Object->Dimensionality;
   for(int i=0;i<Object->Dimensionality;i++)Buffer[i+1]=Object->Dimensions[i];
   SendSADBPacket(Info->Socket,0x00,OUID,Object->DataType,(Object->Time).Second,(Object->Time).Millisecond,
    (Object->Dimensionality+1)*sizeof(u32),(u8 *)Buffer,//Dimension information
    Object->TotalNumberOfElements*Object->ElementSize,(u8*)Object->Data);//Actual Data
   free(Buffer);
   pthread_mutex_unlock(&(Object->Mutex));
  }
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);
}

void SADBServer::CMDGetLatestObjectTimestamp(SADBClientInfo*Info,u64 OUID){
 if(Info->Board->IsValidOUID(OUID)){
  pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));//Make sure that the new value is not being allocated
  if((Info->Board->Objects[OUID-1])->nValues>0){
   SADBObjectStorage * Value=(Info->Board->Objects[OUID-1])->Values[((Info->Board->Objects[OUID-1])->nValues)-1];//latest value
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   pthread_mutex_lock(&(Value->Mutex));
   SendSADBPacket(Info->Socket,0x00,OUID,0,(Value->Time).Second,(Value->Time).Millisecond,0,NULL);
   pthread_mutex_unlock(&(Value->Mutex));
  }else{
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Object has no value
  }
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Object does not exist
}

void SADBServer::CMDGetNextObjectTimestamp(SADBClientInfo*Info,u64 OUID,Timestamp Target){
 if(Info->Board->IsValidOUID(OUID)){
  pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
  if((Info->Board->Objects[OUID-1])->nValues>0){
   SADBObject*Object=Info->Board->Objects[OUID-1];
   SADBObjectStorage*Value;
   //Find the appropriate Value index
   int Position=Object->nValues-1;
   for(int i=0;i<Object->nValues-1;i++)
    if(TimestampDifference(Target,Object->Values[i]->Time)>0){Position=i;break;}
   if(Position<Object->nValues-1)Value=Object->Values[Position];//Not the last value
   else Value=Object->Values[Object->nValues-1];//the last value
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   pthread_mutex_lock(&(Value->Mutex));
   SendSADBPacket(Info->Socket,0x00,OUID,0,(Value->Time).Second,(Value->Time).Millisecond,0,NULL);
   pthread_mutex_unlock(&(Value->Mutex));
  }else{
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Object has no value
  }
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Object does not exist
}

void SADBServer::CMDGetPreviousObjectTimestamp(SADBClientInfo*Info,u64 OUID,Timestamp Target){
 if(Info->Board->IsValidOUID(OUID)){
  pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
  if((Info->Board->Objects[OUID-1])->nValues>0){
   SADBObject*Object=Info->Board->Objects[OUID-1];
   SADBObjectStorage*Value;
   //Find the appropriate Value index
   int Position=Object->nValues-1;
   for(int i=Object->nValues-1;i>=0;i--)
    if(TimestampDifference(Target,Object->Values[i]->Time)<0){Position=i;break;}
   if(Position>0)Value=Object->Values[Position];//Not the first value
   else Value=Object->Values[0];//the first value
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   pthread_mutex_lock(&(Value->Mutex));
   SendSADBPacket(Info->Socket,0x00,OUID,0,(Value->Time).Second,(Value->Time).Millisecond,0,NULL);
   pthread_mutex_unlock(&(Value->Mutex));
  }else{
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Object has no value
  }
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Object does not exist
}

void SADBServer::CMDGetNearestObjectTimestamp(SADBClientInfo*Info,u64 OUID,Timestamp Target){
 if(Info->Board->IsValidOUID(OUID)){
  pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
  if((Info->Board->Objects[OUID-1])->nValues>0){
   SADBObject*Object=Info->Board->Objects[OUID-1];
   SADBObjectStorage*Value;
   //Find the appropriate Value index
   int Position=0;
   for(int i=Object->nValues-1;i>=0;i--){
    int D=TimestampDifference(Object->Values[i]->Time,Target);
    if(D>0){Position=i;break;}
   }
   //Check to see if previous value or next value
   if(Position!=Object->nValues-1){
    int D1=TimestampDifference(Object->Values[Position]->Time,Target);
    int D2=TimestampDifference(Object->Values[Position+1]->Time,Target);
    if(abs(D1)<abs(D2))Value=Object->Values[Position];
    else Value=Object->Values[Position+1];
   }else Value=Object->Values[Object->nValues-1];
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   pthread_mutex_lock(&(Value->Mutex));
   SendSADBPacket(Info->Socket,0x00,OUID,0,(Value->Time).Second,(Value->Time).Millisecond,0,NULL);
   pthread_mutex_unlock(&(Value->Mutex));
  }else{
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Object has no value
  }
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Object does not exist
}

void SADBServer::CMDGetObjectValueAtNearestTimestamp(SADBClientInfo*Info,u64 OUID,Timestamp Target){
 //Send Packet, Argument is the data type, the first 4 bytes of data is the number of dimensions
 //the next n*4 bytes are the dimensions, then the data
 if(Info->Board->IsValidOUID(OUID)){
  pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));//Make sure that the new value is not being allocated
  SADBObject * Object=Info->Board->Objects[OUID-1];
  SADBObjectStorage * Value;
  int Position=0;
  //Find the appropriate Value index
  for(int i=Object->nValues-1;i>=0;i--){
   int D=TimestampDifference(Object->Values[i]->Time,Target);
   if(D>0){Position=i;break;}
  }
  if(Position!=Object->nValues-1){
   int D1=TimestampDifference(Object->Values[Position]->Time,Target);
   int D2=TimestampDifference(Object->Values[Position+1]->Time,Target);
   if(abs(D1)<abs(D2))Value=Object->Values[Position];
   else Value=Object->Values[Position+1];
  }else{
   Value=Object->Values[Object->nValues-1];
  }
  pthread_mutex_lock(&(Value->Mutex));
  pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
  u32 * Buffer=(u32*)malloc((Value->Dimensionality+1)*sizeof(u32));//max of 99 dimensions, can be changed
  Buffer[0]=Value->Dimensionality;
  for(int i=0;i<Value->Dimensionality;i++)
   Buffer[i+1]=Value->Dimensions[i];
  SendSADBPacket(Info->Socket,0x00,OUID,Value->DataType,(Value->Time).Second,(Value->Time).Millisecond,
   (Value->Dimensionality+1)*sizeof(u32),(u8 *)Buffer,//Dimension information
   Value->TotalNumberOfElements*Value->ElementSize,(u8*)Value->Data);//Actual Data
  free(Buffer);
  pthread_mutex_unlock(&(Value->Mutex));
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Response
}

void SADBServer::CMDRemoveOldestValue(SADBClientInfo*Info,u64 OUID){
 if(Info->Board->IsValidOUID(OUID)){
  SADBObject * Object=Info->Board->Objects[OUID-1];
  if(Object->nValues>=1)Object->RemoveValue(0);
 }
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Response
}

void SADBServer::CMDKeepOnlyLatestValues(SADBClientInfo*Info,u64 OUID,int nValues){
 if(Info->Board->IsValidOUID(OUID)){
  SADBObject*Object=Info->Board->Objects[OUID-1];
  if(Object->nValues>=nValues)for(int i=0;i<(Object->nValues-nValues);i++)Object->RemoveValue(i);//Really not optimal, need to rewrite
 }
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Response
}

void SADBServer::CMDGetObjectValueAtTimestamp(SADBClientInfo*Info,u64 OUID,Timestamp Target,u64 Argument){
 if(Info->Board->IsValidOUID(OUID)){
  Interpolator I;
  I.Initialize();
  I.SetObject(Info->Board->Objects[OUID-1]);
  I.GetValue(Target,Argument&0xFF,(Argument&0xFF00)>>8);
  SADBObjectStorage*Value=&I.Data;
  u32 * Buffer=(u32*)malloc((Value->Dimensionality+1)*sizeof(u32));
  Buffer[0]=Value->Dimensionality;
  for(int i=0;i<Value->Dimensionality;i++)
   Buffer[i+1]=Value->Dimensions[i];
  SendSADBPacket(Info->Socket,0x00,OUID,Value->DataType,(Value->Time).Second,(Value->Time).Millisecond,
   (Value->Dimensionality+1)*sizeof(u32),(u8 *)Buffer,//Dimension information
   Value->TotalNumberOfElements*Value->ElementSize,(u8*)Value->Data);//Actual Data
  free(Buffer);
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Response
}

void SADBServer::CMDGetObjectValueAtNextTimestamp(SADBClientInfo*Info,u64 OUID,Timestamp Target){
 //Send Packet, Argument is the data type, the first 4 bytes of data is the number of dimensions
 //the next n*4 bytes are the dimensions, then the data
 if(Info->Board->IsValidOUID(OUID)){
  pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));//Make sure that the new value is not being allocated
  SADBObject * Object=Info->Board->Objects[OUID-1];
  if(Object->nValues==0){//No Stored Values
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);
   return;
  }
  SADBObjectStorage * Value;
  int Position=Object->nValues-1;
  //Find the appropriate Value index
  for(int i=0;i<Object->nValues-1;i++){
   s64 D=TimestampDifference(Target,Object->Values[i]->Time);
   if(D>0){Position=i;break;}
  }
  if(Position<Object->nValues-1){//Not the last value
   Value=Object->Values[Position];
  }else{//the last value
   Value=Object->Values[Object->nValues-1];
  }
  pthread_mutex_lock(&(Value->Mutex));
  pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
  u32 * Buffer=(u32*)malloc((Value->Dimensionality+1)*sizeof(u32));
  Buffer[0]=Value->Dimensionality;
  for(int i=0;i<Value->Dimensionality;i++)Buffer[i+1]=Value->Dimensions[i];
  SendSADBPacket(Info->Socket,0x00,OUID,Value->DataType,(Value->Time).Second,(Value->Time).Millisecond,
   (Value->Dimensionality+1)*sizeof(u32),(u8 *)Buffer,//Dimension information
   Value->TotalNumberOfElements*Value->ElementSize,(u8*)Value->Data);//Actual Data
  free(Buffer);
  pthread_mutex_unlock(&(Value->Mutex));
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Response
}

void SADBServer::CMDGetObjectValueAtPreviousTimestamp(SADBClientInfo*Info,u64 OUID,Timestamp Target){
 //Send Packet, Argument is the data type, the first 4 bytes of data is the number of dimensions
 //the next n*4 bytes are the dimensions, then the data
 if(Info->Board->IsValidOUID(OUID)){
  pthread_mutex_lock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));//Make sure that the new value is not being allocated
  SADBObject * Object=Info->Board->Objects[OUID-1];
  if(Object->nValues==0){//No Stored Values
   pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);
   return;
  }
  SADBObjectStorage * Value;
  int Position=0;
  //Find the appropriate Value index
  for(int i=Object->nValues-1;i>=0;i--){
   s64 D=TimestampDifference(Target,Object->Values[i]->Time);
   if(D<0){Position=i;break;}
  }
  if(Position>0)Value=Object->Values[Position];//Not the last value
  else Value=Object->Values[0];//the first value
  pthread_mutex_lock(&(Value->Mutex));
  pthread_mutex_unlock(&((Info->Board->Objects[OUID-1])->DataAllocationMutex));
  u32 * Buffer=(u32*)malloc((Value->Dimensionality+1)*sizeof(u32));
  Buffer[0]=Value->Dimensionality;
  for(int i=0;i<Value->Dimensionality;i++)Buffer[i+1]=Value->Dimensions[i];
  SendSADBPacket(Info->Socket,0x00,OUID,Value->DataType,(Value->Time).Second,(Value->Time).Millisecond,
   (Value->Dimensionality+1)*sizeof(u32),(u8 *)Buffer,//Dimension information
   Value->TotalNumberOfElements*Value->ElementSize,(u8*)Value->Data);//Actual Data
  free(Buffer);
  pthread_mutex_unlock(&(Value->Mutex));
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Response
}

void SADBServer::CMDGetNextObjectID(SADBClientInfo*Info){
 SendSADBPacket(Info->Socket,0x00,Info->Board->CurrentObjectID,0,0,0,0,0);//Send a response
}

void SADBServer::CMDCreateCategory(SADBClientInfo*Info){
 u64 ID=Info->Board->CreateCategory();
 SendSADBPacket(Info->Socket,0x00,ID,0,0,0,0,NULL);//Send a response
}

void SADBServer::CMDDeleteCategory(SADBClientInfo*Info,u64 CUID){
 Info->Board->RemoveCategory(CUID);
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,NULL);//Send a response
}

void SADBServer::CMDIsValidCategory(SADBClientInfo*Info,u64 CUID){
 if(Info->Board->IsValidCUID(CUID))SendSADBPacket(Info->Socket,0x00,0,1,0,0,0,NULL);
 else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,NULL);
}

void SADBServer::CMDSetCategoryName(SADBClientInfo*Info,u64 CUID,u8 * Name){
 if(Info->Board->IsValidCUID(CUID))(Info->Board->Categories[CUID-1])->SetName((const char *)Name);//Set the name
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,NULL);//Send a response
}

void SADBServer::CMDGetCategoryName(SADBClientInfo*Info,u64 CUID){
 if(Info->Board->IsValidCUID(CUID)){
  pthread_mutex_lock(&((Info->Board->Categories[CUID-1])->InfoMutex));
  if((Info->Board->Categories[CUID-1])->Name==NULL){
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,1,(u8*)"");//Send a response
  }else{
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,
    strlen((const char*)((Info->Board->Categories[CUID-1])->Name))+1,
    ((Info->Board->Categories[CUID-1])->Name) );//Send a response
  }
  pthread_mutex_unlock(&((Info->Board->Categories[CUID-1])->InfoMutex));
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Send a response
}

void SADBServer::CMDSetCategoryDescription(SADBClientInfo*Info,u64 CUID,u8 * Description){
 if(Info->Board->IsValidCUID(CUID))(Info->Board->Categories[CUID-1])->SetDescription((const char *)Description);//Set the name
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,NULL);//Send a response
}

void SADBServer::CMDGetCategoryDescription(SADBClientInfo*Info,u64 CUID){
 if(Info->Board->IsValidCUID(CUID)){
  pthread_mutex_lock(&((Info->Board->Categories[CUID-1])->InfoMutex));
  if((Info->Board->Categories[CUID-1])->Description==NULL){
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,1,(u8*)"");//Send a response
  }else{
   SendSADBPacket(Info->Socket,0x00,0,0,0,0,
    strlen((const char*)((Info->Board->Categories[CUID-1])->Description))+1,
    ((Info->Board->Categories[CUID-1])->Description) );//Send a response
  }
  pthread_mutex_unlock(&((Info->Board->Categories[CUID-1])->InfoMutex));
 }else SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Send a response
}

void SADBServer::CMDGetCategoryByName(SADBClientInfo*Info,u8*Name){
 u64 ID=Info->Board->GetCategoryByName(Name);
 SendSADBPacket(Info->Socket,0x00,ID,0,0,0,0,0);//Send a response
}

void SADBServer::CMDAddObjectToCategory(SADBClientInfo*Info,u64 CUID,u64 OUID){
 //printf("Adding %lx to %lx\n",OUID,CUID);
 if(Info->Board->IsValidCUID(CUID))(Info->Board->Categories[CUID-1])->AddObject(OUID);
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Send a response
}

void SADBServer::CMDRemoveObjectFromCategory(SADBClientInfo*Info,u64 CUID,u64 OUID){
 if(Info->Board->IsValidCUID(CUID))
  (Info->Board->Categories[CUID-1])->RemoveObject(OUID);
 SendSADBPacket(Info->Socket,0x00,0,0,0,0,0,0);//Send a response
}

void SADBServer::CMDGetNextCategoryID(SADBClientInfo*Info){
 SendSADBPacket(Info->Socket,0x00,Info->Board->CurrentCategoryID,0,0,0,0,0);//Send a response
}
