#include "SADB-Client/SADB.h"
using namespace SADB;
#define ServerIP "127.0.0.1"

//General Arguments
char*ServerAddress;//string pointer address, can be ip or hostname
int ServerPort;//port number
Blackboard Board;//SADB Server
char*Filename;//Either output or input filename
int DataType;//Type of data changed to int
 //0=Byte
 //1=Integer
 //2=Float
 //3=Double


void Connect(void){
 printf("Connecting to SADB server at %s on port %d\n",ServerAddress,ServerPort); 
 if(Board.ConnectToServer(ServerAddress,ServerPort)){
  printf("Cannot connect to server\n");exit(0); 
 }
}

void Reset(void){
 printf("Resetting SADB\n");
 Board.Reset();
}

void Help(void){
 printf("Usage:\tSADBCTL [-p ServerPort] [-a ServerAddress] [-f Filename] [-start Percentage] [-end Percentage] [-t DataType] [-h] -action\n"); 
 printf("Action is one of\n");
 printf(" -reset\t: Reset SADB server\n");
 printf(" -load\t: Load content from disk\n");
 printf("   requires -f <Dump folder name>\n");
 printf("   optional -start <Start Percent>\n");
 printf("   optional -end <End Percent>\n");
 printf(" -dump\t: Dump content of SADB to disk\n");
 printf("   requires -f <Dump folder name>\n");
 printf(" -csv\t: Create object with initial value from CSV file\n");
 printf("   requires -f <Filename> and -t <Data Type>\n");
 printf(" -timecsv\t: Create object values from time-coded CSV file\n");
 printf("   requires -f <Filename> and -t <Data Type>\n");
 printf(" -stringtable\t: Create a StringTable object from CSV file\n");
 printf("   requires -f <Filename>\n");
 printf(" -loadlegacy\t: Load content encoded in legacy format from disk\n");
 printf("   requires -f <Dump folder name>\n");
 exit(0);
}

void Dump(void){
 if(Filename==NULL)Help();
 printf("Dumping to %s\n",Filename);
 SADBu64 CUID=Board.GetNextCategoryID();
 SADBu64 OUID=Board.GetNextObjectID();
 char * Temp=(char *)malloc(1000);//Temp buffer to sprintf filenames
 //Create output folder
 sprintf((char *)Temp,"mkdir %s",Filename);
 system(Temp);
 //Create Objects File
 sprintf((char *)Temp,"%s/Objects",Filename);
 FILE *Objects=fopen(Temp,"w");
 //Create Categoriess File
 sprintf((char *)Temp,"%s/Categories",Filename);
 FILE *Categories=fopen(Temp,"w");
/*
 Dump Objects
 File Format:
Name
Description
DataType
Dimensions
*/
 for(SADBu64 i=1;i<OUID;i++){
  if(Board.IsValidObject(i)){
   BlackboardObject T;
   T.SetBlackboard(&Board);
   T.ID=i;
   T.GetName();
   T.GetDescription();
   T.GetLatestValue();
   printf("Dumping: %s\n",T.Name);
   //Write the information to file
   fprintf(Objects,"%lld\n",T.ID);
   fprintf(Objects,"%s\n",T.Name);
   fprintf(Objects,"%s\n",T.Description);
   sprintf((char *)Temp,"%s/%lld",Filename,T.ID);
   T.ExportRaw(Temp);
  }else{
   fprintf(Objects,"0\n");
   fprintf(Objects,"<DELETED>\n");
   fprintf(Objects,"<DELETED>\n");
  }
 }
/*
 Dump Categories
 File Format:
Name
Description
Members
*/
 for(SADBu64 i=1;i<CUID;i++){
  if(Board.IsValidCategory(i)){
   fprintf(Categories,"%lld\n",i);
   unsigned char * Name=NULL;
   Board.GetCategoryName(i,&Name);
   fprintf(Categories,"%s\n",Name);
   unsigned char * Description=NULL;
   Board.GetCategoryDescription(i,&Description);
   fprintf(Categories,"%s\n",Description);
    //Get the list of objects in this category
   SADBu64*List=NULL;
   int n=Board.FindObjects((const char *)Name,&List);
   if(n>0){
    fprintf(Categories,"%lld",List[0]);
    for(int i=1;i<n;i++)fprintf(Categories,",%lld",List[i]);
    fprintf(Categories,"\n");
   }
   free(Name);Name=NULL;
   free(Description);Description=NULL;
   free(List);
  }else{
   fprintf(Categories,"0\n");
   fprintf(Categories,"<DELETED>\n");
   fprintf(Categories,"<DELETED>\n");
   fprintf(Categories,"\n");//objects
  }
 }
 fclose(Objects);
 fclose(Categories);
 free(Temp);
}


void Load(float Start,float End){
 if(Filename==NULL)Help();
 Reset();
 printf("Loading from %s\n",Filename);
 int c;
 char * Temp=(char *)malloc(1000);//Temp buffer to sprintf filenames
 //Create output folder
 sprintf(Temp,"%s/Objects",Filename);
 FILE *Objects=fopen(Temp,"r");
 sprintf(Temp,"%s/Categories",Filename);
 FILE *Categories=fopen(Temp,"r");
 //Load Object Information
 //Count number of objects
 int nLines=0;
 while(c!=EOF){
  c=fgetc(Objects);
  if(c=='\n')nLines++;
 }rewind(Objects);
 int nObjects=nLines/3;//3 lines of info per object
 for(int i=0;i<nObjects;i++){
  BlackboardObject O;
  O.SetBlackboard(&Board);
  O.AddToBlackboard();//Add to blackboard and get ID
  //Check ID
  int j=0;
  while(1){
   c=fgetc(Objects);
   if((c=='\n')||(c==EOF))break;
   Temp[j]=c;
   j++;
  }Temp[j]='\0';
  SADBu64 ID=atoi(Temp);
  if(ID==0){
   Board.DeleteObject(O.ID);
   //Skip next 2 lines
   while(1){
    c=fgetc(Objects);
    if((c=='\n')||(c==EOF))break;
   }
   while(1){
    c=fgetc(Objects);
    if((c=='\n')||(c==EOF))break;
   }
  }else{
   if(ID!=O.ID)Error("Import Object","Something went wrong, are other clients using the db?");
   printf("Object: %lld\n",ID);
   //Set Name
   j=0;
   while(1){
    c=fgetc(Objects);
    if((c=='\n')||(c==EOF))break;
    Temp[j]=c;
    j++;
   }Temp[j]='\0';
   O.SetName(Temp);
   printf(" Name: %s\n",Temp);
   //Set Description
   j=0;
   while(1){
    c=fgetc(Objects);
    if((c=='\n')||(c==EOF))break;
    Temp[j]=c;
    j++;
   }Temp[j]='\0';
   O.SetDescription(Temp);
   printf(" Description: %s\n",Temp);
   //Load Data
   sprintf(Temp,"%s/%lld",Filename,ID);
   O.ImportRaw(Temp,Start,End);
  }
 }
 //Now load categories
 nLines=0;
 while(c!=EOF){
  c=fgetc(Categories);
  if(c=='\n')nLines++;
 }rewind(Categories);
 int nCategories=nLines/4;
 printf("Loading %d Categories\n",nCategories);
 for(int i=0;i<nCategories;i++){
  //Handle ID
  int j=0;
  while(1){
   c=fgetc(Categories);
   if(c=='\n')break;
   Temp[j]=c;
   j++;
  }Temp[j]='\0';
  SADBu64 oldID=atoi(Temp);
  SADBu64 ID=Board.CreateCategory();
  if(oldID==0){
   Board.DeleteCategory(ID);
   //Skip next 3 lines
   while(1){
    c=fgetc(Categories);
    if((c=='\n')||(c==EOF))break;
   }
   while(1){
    c=fgetc(Categories);
    if((c=='\n')||(c==EOF))break;
   }
   while(1){
    c=fgetc(Categories);
    if((c=='\n')||(c==EOF))break;
   }
  }else{
   if(ID!=oldID)Error("Import Category","Something went wrong, are other clients using the db?");
   printf("Category: %lld\n",ID);
   //Handle Name
   j=0;
   while(1){
    c=fgetc(Categories);
    if((c=='\n')||(c==EOF))break;
    Temp[j]=c;
    j++;
   }Temp[j]='\0';
   Board.SetCategoryName(ID,Temp);
   printf(" Name: %s\n",Temp);
   //Handle Description
   j=0;
   while(1){
    c=fgetc(Categories);
    if((c=='\n')||(c==EOF))break;
    Temp[j]=c;
    j++;
   }Temp[j]='\0';
   Board.SetCategoryDescription(ID,Temp);
   printf(" Description: %s\n",Temp);
   //Populate the category
   while(1){
    j=0;
    while(1){
     c=fgetc(Categories);
     if((c=='\n')||(c==EOF)||(c==','))break;
     Temp[j]=c;
     j++;
    }Temp[j]='\0';
    if(j>0)Board.AddObjectToCategory(ID,(SADBu64)atoi(Temp));
    if((c=='\n')||(c==EOF)||(c=='\0'))break;
   }
  } 
 }
 free(Temp);
}


void LoadLegacy(void){
 if(Filename==NULL)Help();
 Reset();
 printf("Loading from %s\n",Filename);
 int c;
 char * Temp=(char *)malloc(1000);//Temp buffer to sprintf filenames
 //Create output folder
 sprintf(Temp,"%s/Objects",Filename);
 FILE *Objects=fopen(Temp,"r");
 sprintf(Temp,"%s/Categories",Filename);
 FILE *Categories=fopen(Temp,"r");
 //Load Object Information
 //Count number of objects
 int nLines=0;
 while(c!=EOF){
  c=fgetc(Objects);
  if(c=='\n')nLines++;
 }rewind(Objects);
 int nObjects=nLines/3;//3 lines of info per object
 for(int i=0;i<nObjects;i++){
  BlackboardObject O;
  O.SetBlackboard(&Board);
  O.AddToBlackboard();//Add to blackboard and get ID
  //Check ID
  int j=0;
  while(1){
   c=fgetc(Objects);
   if((c=='\n')||(c==EOF))break;
   Temp[j]=c;
   j++;
  }Temp[j]='\0';
  SADBu64 ID=atoi(Temp);
  if(ID==0){
   Board.DeleteObject(O.ID);
   //Skip next 2 lines
   while(1){
    c=fgetc(Objects);
    if((c=='\n')||(c==EOF))break;
   }
   while(1){
    c=fgetc(Objects);
    if((c=='\n')||(c==EOF))break;
   }
  }else{
   if(ID!=O.ID)Error("Import Object","Something went wrong, are other clients using the db?");
   printf("Object: %lld\n",ID);
   //Set Name
   j=0;
   while(1){
    c=fgetc(Objects);
    if((c=='\n')||(c==EOF))break;
    Temp[j]=c;
    j++;
   }Temp[j]='\0';
   O.SetName(Temp);
   printf(" Name: %s\n",Temp);
   //Set Description
   j=0;
   while(1){
    c=fgetc(Objects);
    if((c=='\n')||(c==EOF))break;
    Temp[j]=c;
    j++;
   }Temp[j]='\0';
   O.SetDescription(Temp);
   printf(" Description: %s\n",Temp);
   //Load Data
   sprintf(Temp,"%s/%lld",Filename,ID);
   O.ImportAscii(Temp);
  }
 }
 //Now load categories
 nLines=0;
 while(c!=EOF){
  c=fgetc(Categories);
  if(c=='\n')nLines++;
 }rewind(Categories);
 int nCategories=nLines/4;
 printf("Loading %d Categories\n",nCategories);
 for(int i=0;i<nCategories;i++){
  //Handle ID
  int j=0;
  while(1){
   c=fgetc(Categories);
   if(c=='\n')break;
   Temp[j]=c;
   j++;
  }Temp[j]='\0';
  SADBu64 oldID=atoi(Temp);
  SADBu64 ID=Board.CreateCategory();
  if(oldID==0){
   Board.DeleteCategory(ID);
   //Skip next 3 lines
   while(1){
    c=fgetc(Categories);
    if((c=='\n')||(c==EOF))break;
   }
   while(1){
    c=fgetc(Categories);
    if((c=='\n')||(c==EOF))break;
   }
   while(1){
    c=fgetc(Categories);
    if((c=='\n')||(c==EOF))break;
   }
  }else{
   if(ID!=oldID)Error("Import Category","Something went wrong, are other clients using the db?");
   printf("Category: %lld\n",ID);
   //Handle Name
   j=0;
   while(1){
    c=fgetc(Categories);
    if((c=='\n')||(c==EOF))break;
    Temp[j]=c;
    j++;
   }Temp[j]='\0';
   Board.SetCategoryName(ID,Temp);
   printf(" Name: %s\n",Temp);
   //Handle Description
   j=0;
   while(1){
    c=fgetc(Categories);
    if((c=='\n')||(c==EOF))break;
    Temp[j]=c;
    j++;
   }Temp[j]='\0';
   Board.SetCategoryDescription(ID,Temp);
   printf(" Description: %s\n",Temp);
   //Populate the category
   while(1){
    j=0;
    while(1){
     c=fgetc(Categories);
     if((c=='\n')||(c==EOF)||(c==','))break;
     Temp[j]=c;
     j++;
    }Temp[j]='\0';
    if(j>0)Board.AddObjectToCategory(ID,(SADBu64)atoi(Temp));
    if((c=='\n')||(c==EOF)||(c=='\0'))break;
   }
  } 
 }
 free(Temp);
}


void CSVSingleValue(void){
 printf("Loading CSV file: %s of type %d\n",Filename,DataType);
 BlackboardObject Object;
 Object.SetBlackboard(&Board);
 switch(DataType){
  case 0:Object.CreateWithInitialValueFromCSV(Filename,BOByte);break;
  case 1:Object.CreateWithInitialValueFromCSV(Filename,BOInteger);break;
  case 2:Object.CreateWithInitialValueFromCSV(Filename,BOFloat);break;
  case 3:Object.CreateWithInitialValueFromCSV(Filename,BODouble);break;
  default:printf("Wrong data type\n");
 }
}

void CSVMultipleValues(void){
 printf("Loading CSV file: %s of type %d\n",Filename,DataType);
 BlackboardObject Object;
 Object.SetBlackboard(&Board);
 switch(DataType){
  case 0:Object.CreateWithValuesFromCSV(Filename,BOByte);break;
  case 1:Object.CreateWithValuesFromCSV(Filename,BOInteger);break;
  case 2:Object.CreateWithValuesFromCSV(Filename,BOFloat);break;
  case 3:Object.CreateWithValuesFromCSV(Filename,BODouble);break;
  default:printf("Wrong data type\n");
 }
}

void StringTable(void){
 printf("Loading String Table file: %s\n",Filename);
 SADBStringTable Object;
 Object.SetBlackboard(&Board);
 Object.LoadCSV(Filename);
}

int main(int nArgs,char*Args[]){
 ServerPort=SADBPort;//Default port
 char DefaultAddress[]="127.0.0.1";
 ServerAddress=DefaultAddress;//Default to localhost
 DataType=0;//Default to byte
 float Start=0,End=100;
 Filename=NULL;
 int Action=0;
 if(nArgs>1){//There are cmd line args
  for(int i=1;i<nArgs;i++){
   if(Args[i][0]=='-'){
    if((strcmp(Args[i],"-h")==0)||(strcmp(Args[i],"--help")==0))Help();
    if((strcmp(Args[i],"-p")==0)&&(i<nArgs-1)){//Specified Port
     ServerPort=atoi(Args[i+1]);
     if((ServerPort<0)||(ServerPort>65535))ServerPort=SADBPort;//Not valid
    }
    if((strcmp(Args[i],"-a")==0)&&(i<nArgs-1))ServerAddress=Args[i+1];//Specified Address
    if((strcmp(Args[i],"-f")==0)&&(i<nArgs-1))Filename=Args[i+1];//Specified Filename
    if((strcmp(Args[i],"-t")==0)&&(i<nArgs-1)){//Specified Data Type
     if((strcmp(Args[i+1],"Byte")==0)||(strcmp(Args[i+1],"BOByte")==0))DataType=0;
     else if((strcasecmp(Args[i+1],"Integer")==0)||(strcasecmp(Args[i+1],"BOInteger")==0))DataType=1;
     else if((strcasecmp(Args[i+1],"Float")==0)||(strcasecmp(Args[i+1],"BOFloat")==0))DataType=2;
     else if((strcasecmp(Args[i+1],"Double")==0)||(strcasecmp(Args[i+1],"BODouble")==0))DataType=3;
     else DataType=0;//Default to Bytes
    }
    if(strcasecmp(Args[i],"-reset")==0)Action=1;
    if(strcasecmp(Args[i],"-load")==0)Action=2;
    if(strcasecmp(Args[i],"-dump")==0)Action=3;
    if(strcasecmp(Args[i],"-csv")==0)Action=4;
    if(strcasecmp(Args[i],"-timecsv")==0)Action=5;
    if(strcasecmp(Args[i],"-stringtable")==0)Action=6;
    if(strcasecmp(Args[i],"-loadlegacy")==0)Action=7;
    if((strcmp(Args[i],"-start")==0)&&(i<nArgs-1))Start=atof(Args[i+1]);//Specified Start Percentage
    if((strcmp(Args[i],"-end")==0)&&(i<nArgs-1))End=atof(Args[i+1]);//Specified End Percentage
   }
  }
 }else Help();//Needs arguments to run
 Connect();//Connect to the server
 switch(Action){//Actually carry requested action
  case 1:Reset();break;
  case 2:Load(Start,End);break;
  case 3:Dump();break;
  case 4:CSVSingleValue();break;
  case 5:CSVMultipleValues();break;
  case 6:StringTable();break;
  case 7:LoadLegacy();break;
  default:Help();
 }
 exit(0);
}
